@@ -26,201 +26,27 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: channels.h,v $
- * Revision 2.14  2005/11/30 13:35:26  csoutheren
- * Changed tags for Doxygen
- *
- * Revision 2.13  2005/09/04 06:23:38  rjongbloed
- * Added OpalMediaCommand mechanism (via PNotifier) for media streams
- *   and media transcoders to send commands back to remote.
- *
- * Revision 2.12  2005/08/31 13:19:25  rjongbloed
- * Added mechanism for controlling media (especially codecs) including
- *   changing the OpalMediaFormat option list (eg bit rate) and a completely
- *   new OpalMediaCommand abstraction for things like video fast update.
- *
- * Revision 2.11  2002/11/10 11:33:16  robertj
- * Updated to OpenH323 v1.10.3
- *
- * Revision 2.10  2002/09/16 02:52:33  robertj
- * Added #define so can select if #pragma interface/implementation is used on
- *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
- *
- * Revision 2.9  2002/09/04 06:01:46  robertj
- * Updated to OpenH323 v1.9.6
- *
- * Revision 2.8  2002/07/01 04:56:29  robertj
- * Updated to OpenH323 v1.9.1
- *
- * Revision 2.7  2002/04/18 03:41:28  robertj
- * Fixed logical channel so does not delete media stream too early.
- *
- * Revision 2.6  2002/02/11 09:32:11  robertj
- * Updated to openH323 v1.8.0
- *
- * Revision 2.5  2002/02/11 07:37:21  robertj
- * Added media bypass for streams between compatible protocols.
- *
- * Revision 2.4  2002/01/22 04:56:55  robertj
- * Update from OpenH323, rev 1.26 to 1.28
- *
- * Revision 2.3  2002/01/14 06:35:56  robertj
- * Updated to OpenH323 v1.7.9
- *
- * Revision 2.2  2001/11/02 10:45:19  robertj
- * Updated to OpenH323 v1.7.3
- *
- * Revision 2.1  2001/10/15 04:30:09  robertj
- * Added delayed start of media patch threads.
- *
- * Revision 2.0  2001/07/27 15:48:24  robertj
- * Conversion of OpenH323 to Open Phone Abstraction Library (OPAL)
- *
- * Revision 1.40  2002/09/16 01:14:15  robertj
- * Added #define so can select if #pragma interface/implementation is used on
- *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
- *
- * Revision 1.39  2002/09/03 06:19:36  robertj
- * Normalised the multi-include header prevention ifdef/define symbol.
- *
- * Revision 1.38  2002/08/05 10:03:47  robertj
- * Cosmetic changes to normalise the usage of pragma interface/implementation.
- *
- * Revision 1.37  2002/06/25 08:30:08  robertj
- * Changes to differentiate between stright G.723.1 and G.723.1 Annex A using
- *   the OLC dataType silenceSuppression field so does not send SID frames
- *   to receiver codecs that do not understand them.
- *
- * Revision 1.36  2002/05/23 04:53:54  robertj
- * Added function to remove a filter from logical channel.
- *
- * Revision 1.35  2002/05/10 05:46:46  robertj
- * Added session ID to the data logical channel class.
- *
- * Revision 1.34  2002/05/02 07:56:24  robertj
- * Added automatic clearing of call if no media (RTP data) is transferred in a
- *   configurable (default 5 minutes) amount of time.
- *
- * Revision 1.33  2002/05/02 06:28:50  robertj
- * Fixed problem with external RTP channels not fast starting.
- *
- * Revision 1.32  2002/04/17 05:56:28  robertj
- * Added trace output of H323Channel::Direction enum.
- *
- * Revision 1.31  2002/02/09 04:39:01  robertj
- * Changes to allow T.38 logical channels to use single transport which is
- *   now owned by the OpalT38Protocol object instead of H323Channel.
- *
- * Revision 1.30  2002/02/05 08:13:20  robertj
- * Added ability to not have addresses when external RTP channel created.
- *
- * Revision 1.29  2002/01/22 22:48:21  robertj
- * Fixed RFC2833 support (transmitter) requiring large rewrite
- *
- * Revision 1.28  2002/01/17 07:04:57  robertj
- * Added support for RFC2833 embedded DTMF in the RTP stream.
- *
- * Revision 1.27  2002/01/17 00:10:37  robertj
- * Fixed double copy of rtpPayloadType in RTP channel, caused much confusion.
- *
- * Revision 1.26  2002/01/14 05:18:16  robertj
- * Fixed typo on external RTP channel constructor.
- *
- * Revision 1.25  2002/01/10 05:13:50  robertj
- * Added support for external RTP stacks, thanks NuMind Software Systems.
- *
- * Revision 1.24  2001/10/23 02:18:06  dereks
- * Initial release of CU30 video codec.
- *
- * Revision 1.23  2001/07/24 02:26:53  robertj
- * Added start for handling reverse channels.
- *
- * Revision 1.22  2001/07/17 04:44:29  robertj
- * Partial implementation of T.120 and T.38 logical channels.
- *
- * Revision 1.21  2001/02/09 05:16:24  robertj
- * Added #pragma interface for GNU C++.
- *
- * Revision 1.20  2001/01/25 07:27:14  robertj
- * Major changes to add more flexible OpalMediaFormat class to normalise
- *   all information about media types, especially codecs.
- *
- * Revision 1.19  2000/10/19 04:05:20  robertj
- * Added compare function for logical channel numbers, thanks Yuriy Ershov.
- *
- * Revision 1.18  2000/09/22 01:35:02  robertj
- * Added support for handling LID's that only do symmetric codecs.
- *
- * Revision 1.17  2000/08/31 08:15:32  robertj
- * Added support for dynamic RTP payload types in H.245 OpenLogicalChannel negotiations.
- *
- * Revision 1.16  2000/08/21 02:50:19  robertj
- * Fixed race condition if close call just as slow start media channels are opening.
- *
- * Revision 1.15  2000/06/15 01:46:13  robertj
- * Added channel pause (aka mute) functions.
- *
- * Revision 1.14  2000/05/18 11:53:33  robertj
- * Changes to support doc++ documentation generation.
- *
- * Revision 1.13  2000/05/02 04:32:23  robertj
- * Fixed copyright notice comment.
- *
- * Revision 1.12  1999/12/23 23:02:34  robertj
- * File reorganision for separating RTP from H.323 and creation of LID for VPB support.
- *
- * Revision 1.11  1999/11/19 09:06:30  robertj
- * Changed to close down logical channel if get a transmit codec error.
- *
- * Revision 1.10  1999/11/06 05:37:44  robertj
- * Complete rewrite of termination of connection to avoid numerous race conditions.
- *
- * Revision 1.9  1999/09/08 04:05:48  robertj
- * Added support for video capabilities & codec, still needs the actual codec itself!
- *
- * Revision 1.8  1999/08/31 12:34:18  robertj
- * Added gatekeeper support.
- *
- * Revision 1.7  1999/07/09 06:09:49  robertj
- * Major implementation. An ENORMOUS amount of stuff added everywhere.
- *
- * Revision 1.6  1999/06/14 05:15:55  robertj
- * Changes for using RTP sessions correctly in H323 Logical Channel context
- *
- * Revision 1.5  1999/06/13 12:41:14  robertj
- * Implement logical channel transmitter.
- * Fixed H245 connect on receiving call.
- *
- * Revision 1.4  1999/06/09 05:26:19  robertj
- * Major restructuring of classes.
- *
- * Revision 1.3  1999/06/06 06:06:36  robertj
- * Changes for new ASN compiler and v2 protocol ASN files.
- *
- * Revision 1.2  1999/04/26 06:14:46  craigs
- * Initial implementation for RTP decoding and lots of stuff
- * As a whole, these changes are called "First Noise"
- *
- * Revision 1.1  1999/01/16 01:31:07  robertj
- * Initial revision
- *
+ * $Revision: 21293 $
+ * $Author: rjongbloed $
+ * $Date: 2008-10-12 18:24:41 -0500 (Sun, 12 Oct 2008) $
  */
 
-#ifndef __OPAL_CHANNELS_H
-#define __OPAL_CHANNELS_H
+#ifndef OPAL_H323_CHANNELS_H
+#define OPAL_H323_CHANNELS_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
+#include <opal/buildopts.h>
+
+#if OPAL_H323
 
 #include <rtp/rtp.h>
 #include <h323/transaddr.h>
+#include <opal/mediastrm.h>
 
 
-class OpalMediaStream;
-class OpalMediaCommand;
-
 class H245_OpenLogicalChannel;
 class H245_OpenLogicalChannelAck;
 class H245_OpenLogicalChannel_forwardLogicalChannelParameters;
@@ -246,8 +72,8 @@ class H323ChannelNumber : public PObject
   PCLASSINFO(H323ChannelNumber, PObject);
 
   public:
-    H323ChannelNumber() { number = 0; fromRemote = FALSE; }
-    H323ChannelNumber(unsigned number, BOOL fromRemote);
+    H323ChannelNumber() { number = 0; fromRemote = PFalse; }
+    H323ChannelNumber(unsigned number, PBoolean fromRemote);
 
     virtual PObject * Clone() const;
     virtual PINDEX HashFunction() const;
@@ -256,11 +82,11 @@ class H323ChannelNumber : public PObject
 
     H323ChannelNumber & operator++(int);
     operator unsigned() const { return number; }
-    BOOL IsFromRemote() const { return fromRemote; }
+    PBoolean IsFromRemote() const { return fromRemote; }
     
   protected:
     unsigned number;
-    BOOL     fromRemote;
+    PBoolean     fromRemote;
 };
 
 
@@ -327,31 +153,31 @@ class H323Channel : public PObject
        This is primarily used to determine if media bypass is possible for the
        call between two connections.
 
-       The default behaviour returns FALSE.
+       The default behaviour returns PFalse.
      */
-    virtual BOOL GetMediaTransportAddress(
+    virtual PBoolean GetMediaTransportAddress(
       OpalTransportAddress & data,        ///<  Data channel address
       OpalTransportAddress & control      ///<  Control channel address
     ) const;
 
     /**Set the initial bandwidth for the channel.
        This calculates the initial bandwidth required by the channel and
-       returns TRUE if the connection can support this bandwidth.
+       returns PTrue if the connection can support this bandwidth.
 
        The default behaviour gets the bandwidth requirement from the codec
        object created by the channel.
      */
-    virtual BOOL SetInitialBandwidth() = 0;
+    virtual PBoolean SetInitialBandwidth() = 0;
 
     /**Open the channel.
        The default behaviour just calls connection.OnStartLogicalChannel() and
        if successful sets the opened member variable.
       */
-    virtual BOOL Open();
+    virtual PBoolean Open();
 
     /**This is called when the channel can start transferring data.
      */
-    virtual BOOL Start() = 0;
+    virtual PBoolean Start() = 0;
 
     /**This is called to clean up any threads on connection termination.
      */
@@ -359,16 +185,21 @@ class H323Channel : public PObject
 
     /**Indicate if has been opened.
      */
-    BOOL IsOpen() const { return opened && !terminating; }
+    PBoolean IsOpen() const { return opened && !terminating; }
 
     /**Get the media stream associated with this logical channel.
+
+       If the argument is set to PTrue, the mediaStream is about to be deleted
+       so all internal references to the mediaStream must be removed.
+
        The default behaviour returns NULL.
       */
-    virtual OpalMediaStream * GetMediaStream() const = 0;
+    virtual OpalMediaStreamPtr GetMediaStream() const;
+
 
     /**Fill out the OpenLogicalChannel PDU for the particular channel type.
      */
-    virtual BOOL OnSendingPDU(
+    virtual PBoolean OnSendingPDU(
       H245_OpenLogicalChannel & openPDU  ///<  Open PDU to send. 
     ) const = 0;
 
@@ -386,9 +217,9 @@ class H323Channel : public PObject
        local machine via the H245LogicalChannelDict::Open() function, and
        the request has been acknowledged by the remote endpoint.
 
-       The default behaviour just returns TRUE.
+       The default behaviour just returns PTrue.
      */
-    virtual BOOL OnReceivedPDU(
+    virtual PBoolean OnReceivedPDU(
       const H245_OpenLogicalChannel & pdu,    ///<  Open PDU
       unsigned & errorCode                    ///<  Error code on failure
     );
@@ -397,9 +228,9 @@ class H323Channel : public PObject
        local machine via the H245LogicalChannelDict::Open() function, and
        the request has been acknowledged by the remote endpoint.
 
-       The default behaviour just returns TRUE.
+       The default behaviour just returns PTrue.
      */
-    virtual BOOL OnReceivedAckPDU(
+    virtual PBoolean OnReceivedAckPDU(
       const H245_OpenLogicalChannelAck & pdu  ///<  Acknowledgement PDU
     );
 
@@ -458,7 +289,7 @@ class H323Channel : public PObject
 
     /**Get the bandwidth used by the channel in 100's of bits/sec.
      */
-    BOOL SetBandwidthUsed(
+    PBoolean SetBandwidthUsed(
       unsigned bandwidth  ///<  New bandwidth
     );
 
@@ -474,7 +305,7 @@ class H323Channel : public PObject
        Note that channel is not stopped, and may continue to actually receive
        data, it is just that nothing is done with it.
       */
-    BOOL IsPaused() const { return paused; }
+    PBoolean IsPaused() const { return paused; }
 
     /**Set the "pause" flag.
        A paused channel is one that prevents the annunciation of the channels
@@ -485,19 +316,21 @@ class H323Channel : public PObject
        data, it is just that nothing is done with it.
       */
     void SetPause(
-      BOOL pause   ///<  New pause flag
+      PBoolean pause   ///<  New pause flag
     ) { paused = pause; }
   //@}
 
+    virtual void OnMediaCommand(OpalMediaCommand &) { }
+
   protected:
     H323EndPoint         & endpoint;
     H323Connection       & connection;
     H323Capability       * capability;
     H323ChannelNumber      number;
     H323ChannelNumber      reverseChannel;
-    BOOL                   opened;
-    BOOL                   paused;
-    BOOL                   terminating;
+    PBoolean                   opened;
+    PBoolean                   paused;
+    PBoolean                   terminating;
 
   private:
     unsigned bandwidthUsed;
@@ -544,22 +377,22 @@ class H323UnidirectionalChannel : public H323Channel
 
     /**Set the initial bandwidth for the channel.
        This calculates the initial bandwidth required by the channel and
-       returns TRUE if the connection can support this bandwidth.
+       returns PTrue if the connection can support this bandwidth.
 
        The default behaviour gets the bandwidth requirement from the codec
        object created by the channel.
      */
-    virtual BOOL SetInitialBandwidth();
+    virtual PBoolean SetInitialBandwidth();
 
     /**Open the channel.
       */
-    virtual BOOL Open();
+    virtual PBoolean Open();
 
     /**This is called when the channel can start transferring data.
        The default action is to start one threads, with it either calling
        Receive() or Transmit() depending on the receiver member variable.
      */
-    virtual BOOL Start();
+    virtual PBoolean Start();
 
     /**This is called to clean up any threads on connection termination.
      */
@@ -578,14 +411,14 @@ class H323UnidirectionalChannel : public H323Channel
     /**Get the media stream associated with this logical channel.
        The default behaviour returns NULL.
       */
-    virtual OpalMediaStream * GetMediaStream() const;
+    virtual OpalMediaStreamPtr GetMediaStream() const;
   //@}
 
-  protected:
-    PDECLARE_NOTIFIER(OpalMediaCommand, H323UnidirectionalChannel, OnMediaCommand);
+    void OnMediaCommand(OpalMediaCommand & command);
 
-    BOOL              receiver;
-    OpalMediaStream * mediaStream;
+  protected:
+    bool               receiver;
+    OpalMediaStreamPtr mediaStream;
 };
 
 
@@ -622,7 +455,7 @@ class H323BidirectionalChannel : public H323Channel
        The default action is to start two threads, one calls Receive() and the
        other calls Transmit().
      */
-    virtual BOOL Start();
+    virtual PBoolean Start();
   //@}
 };
 
@@ -651,7 +484,7 @@ class H323_RealTimeChannel : public H323UnidirectionalChannel
   //@{
     /**Fill out the OpenLogicalChannel PDU for the particular channel type.
      */
-    virtual BOOL OnSendingPDU(
+    virtual PBoolean OnSendingPDU(
       H245_OpenLogicalChannel & openPDU  ///<  Open PDU to send. 
     ) const;
 
@@ -670,7 +503,7 @@ class H323_RealTimeChannel : public H323UnidirectionalChannel
        The default makes sure the parameters are compatible and passes on
        the PDU to the rtp session.
      */
-    virtual BOOL OnReceivedPDU(
+    virtual PBoolean OnReceivedPDU(
       const H245_OpenLogicalChannel & pdu,    ///<  Open PDU
       unsigned & errorCode                    ///<  Error code on failure
     );
@@ -682,7 +515,7 @@ class H323_RealTimeChannel : public H323UnidirectionalChannel
        The default makes sure the parameters are compatible and passes on
        the PDU to the rtp session.
      */
-    virtual BOOL OnReceivedAckPDU(
+    virtual PBoolean OnReceivedAckPDU(
       const H245_OpenLogicalChannelAck & pdu ///<  Acknowledgement PDU
     );
   //@}
@@ -691,7 +524,7 @@ class H323_RealTimeChannel : public H323UnidirectionalChannel
   //@{
     /**Fill out the OpenLogicalChannel PDU for the particular channel type.
      */
-    virtual BOOL OnSendingPDU(
+    virtual PBoolean OnSendingPDU(
       H245_H2250LogicalChannelParameters & param  ///<  Open PDU to send.
     ) const = 0;
 
@@ -708,7 +541,7 @@ class H323_RealTimeChannel : public H323UnidirectionalChannel
 
        The default behaviour sets the remote ports to send UDP packets to.
      */
-    virtual BOOL OnReceivedPDU(
+    virtual PBoolean OnReceivedPDU(
       const H245_H2250LogicalChannelParameters & param, ///<  Acknowledgement PDU
       unsigned & errorCode                              ///<  Error on failure
     ) = 0;
@@ -719,13 +552,13 @@ class H323_RealTimeChannel : public H323UnidirectionalChannel
 
        The default behaviour sets the remote ports to send UDP packets to.
      */
-    virtual BOOL OnReceivedAckPDU(
+    virtual PBoolean OnReceivedAckPDU(
       const H245_H2250LogicalChannelAckParameters & param ///<  Acknowledgement PDU
     ) = 0;
 
     /**Set the dynamic payload type used by this channel.
       */
-    virtual BOOL SetDynamicRTPPayloadType(
+    virtual PBoolean SetDynamicRTPPayloadType(
       int newType  ///<  New RTP payload type number
     );
 
@@ -774,7 +607,7 @@ class H323_RTPChannel : public H323_RealTimeChannel
   //@{
     /**Fill out the OpenLogicalChannel PDU for the particular channel type.
      */
-    virtual BOOL OnSendingPDU(
+    virtual PBoolean OnSendingPDU(
       H245_H2250LogicalChannelParameters & param  ///<  Open PDU to send.
     ) const;
 
@@ -791,7 +624,7 @@ class H323_RTPChannel : public H323_RealTimeChannel
 
        The default behaviour sets the remote ports to send UDP packets to.
      */
-    virtual BOOL OnReceivedPDU(
+    virtual PBoolean OnReceivedPDU(
       const H245_H2250LogicalChannelParameters & param, ///<  Acknowledgement PDU
       unsigned & errorCode                              ///<  Error on failure
     );
@@ -802,7 +635,7 @@ class H323_RTPChannel : public H323_RealTimeChannel
 
        The default behaviour sets the remote ports to send UDP packets to.
      */
-    virtual BOOL OnReceivedAckPDU(
+    virtual PBoolean OnReceivedAckPDU(
       const H245_H2250LogicalChannelAckParameters & param ///<  Acknowledgement PDU
     );
   //@}
@@ -867,16 +700,16 @@ class H323_ExternalRTPChannel : public H323_RealTimeChannel
        This is primarily used to determine if media bypass is possible for the
        call between two connections.
 
-       The default behaviour returns FALSE.
+       The default behaviour returns PFalse.
      */
-    virtual BOOL GetMediaTransportAddress(
+    virtual PBoolean GetMediaTransportAddress(
       OpalTransportAddress & data,        ///<  Data channel address
       OpalTransportAddress & control      ///<  Control channel address
     ) const;
 
     /**Start the channel.
       */
-    virtual BOOL Start();
+    virtual PBoolean Start();
 
     /**Handle channel data reception.
 
@@ -899,7 +732,7 @@ class H323_ExternalRTPChannel : public H323_RealTimeChannel
   //@{
     /**Fill out the OpenLogicalChannel PDU for the particular channel type.
      */
-    virtual BOOL OnSendingPDU(
+    virtual PBoolean OnSendingPDU(
       H245_H2250LogicalChannelParameters & param  ///<  Open PDU to send.
     ) const;
 
@@ -916,7 +749,7 @@ class H323_ExternalRTPChannel : public H323_RealTimeChannel
 
        The default behaviour sets the remote ports to send UDP packets to.
      */
-    virtual BOOL OnReceivedPDU(
+    virtual PBoolean OnReceivedPDU(
       const H245_H2250LogicalChannelParameters & param, ///<  Acknowledgement PDU
       unsigned & errorCode                              ///<  Error on failure
     );
@@ -927,7 +760,7 @@ class H323_ExternalRTPChannel : public H323_RealTimeChannel
 
        The default behaviour sets the remote ports to send UDP packets to.
      */
-    virtual BOOL OnReceivedAckPDU(
+    virtual PBoolean OnReceivedAckPDU(
       const H245_H2250LogicalChannelAckParameters & param ///<  Acknowledgement PDU
     );
   //@}
@@ -940,13 +773,13 @@ class H323_ExternalRTPChannel : public H323_RealTimeChannel
     const H323TransportAddress & GetRemoteMediaAddress()        const { return remoteMediaAddress; }
     const H323TransportAddress & GetRemoteMediaControlAddress() const { return remoteMediaControlAddress; }
 
-    BOOL GetRemoteAddress(
+    PBoolean GetRemoteAddress(
       PIPSocket::Address & ip,
       WORD & dataPort
     ) const;
 
   protected:
-    void Construct(unsigned id);
+    void Construct(H323Connection & conn, unsigned id);
 
     unsigned             sessionID;
     H323TransportAddress externalMediaAddress;
@@ -999,7 +832,7 @@ class H323DataChannel : public H323UnidirectionalChannel
 
     /**Fill out the OpenLogicalChannel PDU for the particular channel type.
      */
-    virtual BOOL OnSendingPDU(
+    virtual PBoolean OnSendingPDU(
       H245_OpenLogicalChannel & openPDU  ///<  Open PDU to send. 
     ) const;
 
@@ -1018,7 +851,7 @@ class H323DataChannel : public H323UnidirectionalChannel
        The default makes sure the parameters are compatible and passes on
        the PDU to the rtp session.
      */
-    virtual BOOL OnReceivedPDU(
+    virtual PBoolean OnReceivedPDU(
       const H245_OpenLogicalChannel & pdu,    ///<  Open PDU
       unsigned & errorCode                    ///<  Error code on failure
     );
@@ -1030,7 +863,7 @@ class H323DataChannel : public H323UnidirectionalChannel
        The default makes sure the parameters are compatible and passes on
        the PDU to the rtp session.
      */
-    virtual BOOL OnReceivedAckPDU(
+    virtual PBoolean OnReceivedAckPDU(
       const H245_OpenLogicalChannelAck & pdu ///<  Acknowledgement PDU
     );
   //@}
@@ -1041,31 +874,33 @@ class H323DataChannel : public H323UnidirectionalChannel
        This is called on receipt of an OpenLogicalChannel request.
 
        The default behaviour creates a compatible listener using the
-       connections control channel as a basis and returns TRUE if successful.
+       connections control channel as a basis and returns PTrue if successful.
       */
-    virtual BOOL CreateListener();
+    virtual PBoolean CreateListener();
 
     /**Create the H323Transport class to be used.
        This is called on receipt of an OpenLogicalChannelAck response. It
-       should not return TRUE unless the transport member variable is set.
+       should not return PTrue unless the transport member variable is set.
 
        The default behaviour uses the connection signalling channel to create
-       the transport and returns TRUE if successful.
+       the transport and returns PTrue if successful.
       */
-    virtual BOOL CreateTransport();
+    virtual PBoolean CreateTransport();
   //@}
 
   protected:
     unsigned        sessionID;
     H323Listener  * listener;
-    BOOL            autoDeleteListener;
+    PBoolean            autoDeleteListener;
     H323Transport * transport;
-    BOOL            autoDeleteTransport;
-    BOOL            separateReverseChannel;
+    PBoolean            autoDeleteTransport;
+    PBoolean            separateReverseChannel;
 };
 
 
-#endif // __OPAL_CHANNELS_H
+#endif // OPAL_H323
+
+#endif // OPAL_H323_CHANNELS_H
 
 
 /////////////////////////////////////////////////////////////////////////////
