@@ -23,112 +23,21 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: peclient.h,v $
- * Revision 2.3  2005/11/30 13:35:26  csoutheren
- * Changed tags for Doxygen
- *
- * Revision 2.2  2004/06/04 06:54:17  csoutheren
- * Migrated updates from OpenH323 1.14.1
- *
- * Revision 2.1  2004/02/19 10:46:44  rjongbloed
- * Merged OpenH323 version 1.13.1 changes.
- *
- * Revision 1.28  2003/05/14 03:04:58  rjongbloed
- * Added another method for removing service relationships.
- * Added virtual for handling SR requests.
- *
- * Revision 1.27  2003/05/05 08:28:45  craigs
- * Fixed lastUpdate time in descriptors
- *
- * Revision 1.26  2003/04/30 04:56:57  craigs
- * Improved handling for nonexistent routes
- *
- * Revision 1.25  2003/04/10 12:37:54  craigs
- * Improved handling of wildcard entries
- *
- * Revision 1.24  2003/04/10 07:05:15  craigs
- * Allowed access to endpoint type in descriptors
- *
- * Revision 1.23  2003/04/10 03:42:16  craigs
- * Allow AccessRequest to return multiple transport addresses
- *
- * Revision 1.22  2003/04/09 10:47:25  craigs
- * Fixed problems
- *
- * Revision 1.21  2003/04/08 12:23:54  craigs
- * Fixed problem with descriptors not being removed when service relationships go away
- *
- * Revision 1.20  2003/04/07 05:11:13  craigs
- * Added changes to get access to descriptor creates/updates/deletes
- *
- * Revision 1.19  2003/04/02 06:05:10  robertj
- * Added versions of AddDescriptor that contain the GUID.
- *
- * Revision 1.18  2003/04/01 05:59:30  robertj
- * Fixed H.501 transaction code setting members for m_common PDU part.
- *
- * Revision 1.17  2003/04/01 04:47:48  robertj
- * Abstracted H.225 RAS transaction processing (RIP and secondary thread) in
- *   server environment for use by H.501 peer elements.
- *
- * Revision 1.16  2003/04/01 01:17:44  robertj
- * Minor changes to AccessEquest and AddDescriptor API to allow for
- *   string arrays of aliases.
- *
- * Revision 1.15  2003/03/28 00:30:23  craigs
- * Fixed problems with service relationship ordinals and better descriptor update access
- *
- * Revision 1.14  2003/03/27 09:24:06  craigs
- * Rewritten support for descriptors and multiple templates
- *
- * Revision 1.13  2003/03/26 07:08:46  robertj
- * Added default parameters to AddDescriptor() functions
- *
- * Revision 1.12  2003/03/25 07:50:11  craigs
- * Added support for mutiple transports per descriptor
- *
- * Revision 1.11  2003/03/25 05:13:13  craigs
- * More speed enhancements
- *
- * Revision 1.10  2003/03/25 01:58:46  robertj
- * Fixed GNU warnings
- *
- * Revision 1.9  2003/03/25 01:47:58  craigs
- * Changes for new OpenH323 H.501
- *
- * Revision 1.8  2003/03/20 01:51:07  robertj
- * More abstraction of H.225 RAS and H.501 protocols transaction handling.
- *
- * Revision 1.7  2003/03/18 13:58:16  craigs
- * More H.501 implementation
- *
- * Revision 1.6  2003/03/18 02:18:45  craigs
- * Changed incorrect include
- *
- * Revision 1.5  2003/03/17 13:19:49  craigs
- * More H501 implementation
- *
- * Revision 1.4  2003/03/14 06:00:42  craigs
- * More updates
- *
- * Revision 1.3  2003/03/01 00:23:51  craigs
- * New PeerElement implementation
- *
- * Revision 1.2  2003/02/21 07:23:18  robertj
- * Fixed up some comments
- *
- * Revision 1.1  2003/02/21 05:28:39  craigs
- * Factored out code for user with peer elements
- *
+ * $Revision: 21293 $
+ * $Author: rjongbloed $
+ * $Date: 2008-10-12 18:24:41 -0500 (Sun, 12 Oct 2008) $
  */
 
-#ifndef __OPAL_PECLIENT_H
-#define __OPAL_PECLIENT_H
+#ifndef OPAL_H323_PECLIENT_H
+#define OPAL_H323_PECLIENT_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
+#include <opal/buildopts.h>
+
+#if OPAL_H501
 
 #include <h323/h323annexg.h>
 #include <h323/h323ep.h>
@@ -148,7 +57,7 @@ class H501Transaction : public H323Transaction
     H501Transaction(
       H323PeerElement & pe,
       const H501PDU & pdu,
-      BOOL hasReject
+      PBoolean hasReject
     );
 
     virtual H323TransactionPDU * CreateRIP(
@@ -276,10 +185,10 @@ class H323PeerElementDescriptor : public PSafeObject
     };
 
     static inline unsigned SetPriorityOption(unsigned pri)     { return Option_PrioritySet | ((pri & 0x7f) << 6); }
-    static inline unsigned GetPriorityOption(unsigned options) { return (options & Option_PrioritySet) ? ((options >> 6) & 0x7f) : DefaultPriority; }
+    static inline unsigned GetPriorityOption(unsigned options) { return (options & Option_PrioritySet) != 0 ? ((options >> 6) & 0x7f) : (int)DefaultPriority; }
 
     void CopyTo(H501_Descriptor & descriptor);
-    static BOOL CopyToAddressTemplate(H501_AddressTemplate & addressTemplates, 
+    static PBoolean CopyToAddressTemplate(H501_AddressTemplate & addressTemplates, 
                                    const H225_EndpointType & ep, 
                            const H225_ArrayOf_AliasAddress & aliases, 
                            const H225_ArrayOf_AliasAddress & transportAddress, 
@@ -290,7 +199,7 @@ class H323PeerElementDescriptor : public PSafeObject
 
     OpalGloballyUniqueID descriptorID;
 
-    BOOL ContainsNonexistent();
+    PBoolean ContainsNonexistent();
 
     enum States {
       Clean,
@@ -405,12 +314,12 @@ class H323PeerElement : public H323_AnnexG
 
     /**Send a service request
      */
-    BOOL SetOnlyServiceRelationship(const PString & peer, BOOL keepTrying = TRUE);
-    BOOL AddServiceRelationship(const H323TransportAddress & peer, BOOL keepTrying = TRUE);
-    BOOL AddServiceRelationship(const H323TransportAddress & peer, OpalGloballyUniqueID & serviceID, BOOL keepTrying = TRUE);
-    BOOL RemoveServiceRelationship(const OpalGloballyUniqueID & serviceID, int reason = H501_ServiceReleaseReason::e_terminated);
-    BOOL RemoveServiceRelationship(const H323TransportAddress & peer, int reason = H501_ServiceReleaseReason::e_terminated);
-    BOOL RemoveAllServiceRelationships();
+    PBoolean SetOnlyServiceRelationship(const PString & peer, PBoolean keepTrying = PTrue);
+    PBoolean AddServiceRelationship(const H323TransportAddress & peer, PBoolean keepTrying = PTrue);
+    PBoolean AddServiceRelationship(const H323TransportAddress & peer, OpalGloballyUniqueID & serviceID, PBoolean keepTrying = PTrue);
+    PBoolean RemoveServiceRelationship(const OpalGloballyUniqueID & serviceID, int reason = H501_ServiceReleaseReason::e_terminated);
+    PBoolean RemoveServiceRelationship(const H323TransportAddress & peer, int reason = H501_ServiceReleaseReason::e_terminated);
+    PBoolean RemoveAllServiceRelationships();
 
     Error ServiceRequestByAddr(const H323TransportAddress & peer);
     Error ServiceRequestByAddr(const H323TransportAddress & peer, OpalGloballyUniqueID & serviceID);
@@ -418,83 +327,83 @@ class H323PeerElement : public H323_AnnexG
 
     /**Send a service release
      */
-    BOOL ServiceRelease(const OpalGloballyUniqueID & serviceID, unsigned reason);
+    PBoolean ServiceRelease(const OpalGloballyUniqueID & serviceID, unsigned reason);
 
     /*********************************************************
       functions to manipulate the local descriptor table
      */
 
-    BOOL AddDescriptor(
+    PBoolean AddDescriptor(
       const OpalGloballyUniqueID & descriptorID,
       const PStringArray & aliases, 
       const H323TransportAddressArray & transportAddrs, 
       unsigned options = H323PeerElementDescriptor::Protocol_H323, 
-      BOOL now = FALSE
+      PBoolean now = PFalse
     );
 
-    BOOL AddDescriptor(
+    PBoolean AddDescriptor(
       const OpalGloballyUniqueID & descriptorID,
       const H225_ArrayOf_AliasAddress & aliases, 
       const H323TransportAddressArray & transportAddrs, 
       unsigned options = H323PeerElementDescriptor::Protocol_H323, 
-      BOOL now = FALSE
+      PBoolean now = PFalse
     );
 
-    BOOL AddDescriptor(
+    PBoolean AddDescriptor(
       const OpalGloballyUniqueID & descriptorID,
       const H225_ArrayOf_AliasAddress & aliases, 
       const H225_ArrayOf_AliasAddress & transportAddr, 
       unsigned options = H323PeerElementDescriptor::Protocol_H323, 
-      BOOL now = FALSE
+      PBoolean now = PFalse
     );
 
-    BOOL AddDescriptor(
+    PBoolean AddDescriptor(
       const OpalGloballyUniqueID & descriptorID,
       const POrdinalKey & creator,
       const H225_ArrayOf_AliasAddress & alias, 
       const H225_ArrayOf_AliasAddress & transportAddresses,
       unsigned options = H323PeerElementDescriptor::Protocol_H323,
-      BOOL now = FALSE
+      PBoolean now = PFalse
     );
 
-    BOOL AddDescriptor(
+    PBoolean AddDescriptor(
       const OpalGloballyUniqueID & descriptorID,
       const POrdinalKey & creator,
       const H501_ArrayOf_AddressTemplate & addressTemplates,
       const PTime & updateTime,
-      BOOL now = FALSE
+      PBoolean now = PFalse
     );
 
     /**Remove a descriptor from the local table
      */
-    BOOL DeleteDescriptor(const PString & alias, BOOL now = FALSE);
-    BOOL DeleteDescriptor(const H225_AliasAddress & alias, BOOL now = FALSE);
-    BOOL DeleteDescriptor(const OpalGloballyUniqueID & descriptorID, BOOL now = FALSE);
+    PBoolean DeleteDescriptor(const PString & alias, PBoolean now = PFalse);
+    PBoolean DeleteDescriptor(const H225_AliasAddress & alias, PBoolean now = PFalse);
+    PBoolean DeleteDescriptor(const OpalGloballyUniqueID & descriptorID, PBoolean now = PFalse);
 
     /** Request access to an alias
     */
-    BOOL AccessRequest(
+    PBoolean AccessRequest(
       const PString & searchAlias,
       PStringArray & destAliases, 
       H323TransportAddress & transportAddress,
       unsigned options = H323PeerElementDescriptor::Protocol_H323
     );
 
-    BOOL AccessRequest(
+    PBoolean AccessRequest(
       const PString & searchAlias,
       H225_ArrayOf_AliasAddress & destAliases,
       H323TransportAddress & transportAddress,
       unsigned options = H323PeerElementDescriptor::Protocol_H323
     );
 
-    BOOL AccessRequest(
+    PBoolean AccessRequest(
       const H225_AliasAddress & searchAlias,
       H225_ArrayOf_AliasAddress & destAliases,
       H323TransportAddress & transportAddress,
       unsigned options = H323PeerElementDescriptor::Protocol_H323
     );
 
-    BOOL AccessRequest(
+    PBoolean AccessRequest(
       const H225_AliasAddress & alias,
       H225_ArrayOf_AliasAddress & destAliases,
       H225_AliasAddress & transportAddress,
@@ -504,8 +413,8 @@ class H323PeerElement : public H323_AnnexG
     /*********************************************************
       functions to send send descriptors to another peer element
       */
-    BOOL UpdateDescriptor(H323PeerElementDescriptor * descriptor);
-    BOOL UpdateDescriptor(H323PeerElementDescriptor * descriptor, H501_UpdateInformation_updateType::Choices updateType);
+    PBoolean UpdateDescriptor(H323PeerElementDescriptor * descriptor);
+    PBoolean UpdateDescriptor(H323PeerElementDescriptor * descriptor, H501_UpdateInformation_updateType::Choices updateType);
 
 
     Error SendUpdateDescriptorByID(const OpalGloballyUniqueID & serviceID, 
@@ -529,7 +438,7 @@ class H323PeerElement : public H323_AnnexG
       low level request functions
       */
 
-    BOOL MakeRequest(H323_AnnexG::Request & request);
+    PBoolean MakeRequest(H323_AnnexG::Request & request);
 
     virtual void OnAddServiceRelationship(const H323TransportAddress &) { }
     virtual void OnRemoveServiceRelationship(const H323TransportAddress &) { }
@@ -542,26 +451,26 @@ class H323PeerElement : public H323_AnnexG
     virtual H323Transaction::Response OnDescriptorUpdate(H501DescriptorUpdate & info);
     virtual H323Transaction::Response OnAccessRequest(H501AccessRequest & info);
 
-    BOOL OnReceiveServiceRequest(const H501PDU & pdu, const H501_ServiceRequest & pduBody);
-    BOOL OnReceiveServiceConfirmation(const H501PDU & pdu, const H501_ServiceConfirmation & pduBody);
+    PBoolean OnReceiveServiceRequest(const H501PDU & pdu, const H501_ServiceRequest & pduBody);
+    PBoolean OnReceiveServiceConfirmation(const H501PDU & pdu, const H501_ServiceConfirmation & pduBody);
 
-    BOOL OnReceiveDescriptorUpdate(const H501PDU & pdu, const H501_DescriptorUpdate & pduBody);
-    BOOL OnReceiveDescriptorUpdateACK(const H501PDU & pdu, const H501_DescriptorUpdateAck & pduBody);
+    PBoolean OnReceiveDescriptorUpdate(const H501PDU & pdu, const H501_DescriptorUpdate & pduBody);
+    PBoolean OnReceiveDescriptorUpdateACK(const H501PDU & pdu, const H501_DescriptorUpdateAck & pduBody);
 
-    BOOL OnReceiveAccessRequest(const H501PDU & pdu, const H501_AccessRequest & pduBody);
-    BOOL OnReceiveAccessConfirmation (const H501PDU & pdu, const H501_AccessConfirmation & pduBody);
-    BOOL OnReceiveAccessRejection(const H501PDU & pdu,     const H501_AccessRejection & pduBody);
+    PBoolean OnReceiveAccessRequest(const H501PDU & pdu, const H501_AccessRequest & pduBody);
+    PBoolean OnReceiveAccessConfirmation (const H501PDU & pdu, const H501_AccessConfirmation & pduBody);
+    PBoolean OnReceiveAccessRejection(const H501PDU & pdu,     const H501_AccessRejection & pduBody);
 
     class AliasKey : public H225_AliasAddress
     {
       public:
-        AliasKey(const H225_AliasAddress & _alias, const OpalGloballyUniqueID & _id, PINDEX _pos, BOOL _wild = FALSE)
+        AliasKey(const H225_AliasAddress & _alias, const OpalGloballyUniqueID & _id, PINDEX _pos, PBoolean _wild = PFalse)
           : H225_AliasAddress(_alias), id(_id), pos(_pos), wild(_wild)
         { }
 
         OpalGloballyUniqueID id;
         PINDEX pos;
-        BOOL wild;
+        PBoolean wild;
     };
 
   protected:
@@ -572,13 +481,13 @@ class H323PeerElement : public H323_AnnexG
                            H323PeerElementDescriptor * descriptor,
             H501_UpdateInformation_updateType::Choices updateType);
 
-    BOOL OnRemoteServiceRelationshipDisappeared(OpalGloballyUniqueID & serviceID, const H323TransportAddress & peer);
+    PBoolean OnRemoteServiceRelationshipDisappeared(OpalGloballyUniqueID & serviceID, const H323TransportAddress & peer);
     void InternalRemoveServiceRelationship(const H323TransportAddress & peer);
     H323Transaction::Response HandleServiceRequest(H501ServiceRequest & info);
 
     virtual H323PeerElementDescriptor          * CreateDescriptor(const OpalGloballyUniqueID & descriptorID);
     virtual H323PeerElementServiceRelationship * CreateServiceRelationship();
-    virtual AliasKey                           * CreateAliasKey(const H225_AliasAddress & alias, const OpalGloballyUniqueID & id, PINDEX pos, BOOL wild = FALSE);
+    virtual AliasKey                           * CreateAliasKey(const H225_AliasAddress & alias, const OpalGloballyUniqueID & id, PINDEX pos, PBoolean wild = PFalse);
 
     void RemoveDescriptorInformation(const H501_ArrayOf_AddressTemplate & addressTemplates);
 
@@ -592,7 +501,7 @@ class H323PeerElement : public H323_AnnexG
 
     PSemaphore requestMutex;
     PThread  * monitor;
-    BOOL       monitorStop;
+    PBoolean       monitorStop;
     PSyncPoint monitorTickle;
 
     PMutex basePeerOrdinalMutex;
@@ -621,7 +530,9 @@ class H323PeerElement : public H323_AnnexG
 };
 
 
-#endif // __OPAL_PECLIENT_H
+#endif // OPAL_H501
+
+#endif // OPAL_H323_PECLIENT_H
 
 
 /////////////////////////////////////////////////////////////////////////////
