@@ -24,113 +24,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: endpoint.h,v $
- * Revision 2.29  2005/11/30 13:35:26  csoutheren
- * Changed tags for Doxygen
- *
- * Revision 2.28  2005/10/08 19:26:37  dsandras
- * Added OnForwarded callback in case of call forwarding.
- *
- * Revision 2.27  2005/08/24 10:43:51  rjongbloed
- * Changed create video functions yet again so can return pointers that are not to be deleted.
- *
- * Revision 2.26  2005/08/23 12:45:09  rjongbloed
- * Fixed creation of video preview window and setting video grab/display initial frame size.
- *
- * Revision 2.25  2005/07/14 08:51:18  csoutheren
- * Removed CreateExternalRTPAddress - it's not needed because you can override GetMediaAddress
- * to do the same thing
- * Fixed problems with logic associated with media bypass
- *
- * Revision 2.24  2005/07/11 06:52:16  csoutheren
- * Added support for outgoing calls using external RTP
- *
- * Revision 2.23  2005/07/11 01:52:23  csoutheren
- * Extended AnsweringCall to work for SIP as well as H.323
- * Fixed problems with external RTP connection in H.323
- * Added call to OnClosedMediaStream
- *
- * Revision 2.22  2005/04/10 20:45:22  dsandras
- * Added callback that is called when a connection is put on hold (locally or remotely).
- *
- * Revision 2.21  2004/08/14 07:56:29  rjongbloed
- * Major revision to utilise the PSafeCollection classes for the connections and calls.
- *
- * Revision 2.20  2004/07/11 12:42:10  rjongbloed
- * Added function on endpoints to get the list of all media formats any
- *   connection the endpoint may create can support.
- *
- * Revision 2.19  2004/04/26 06:30:32  rjongbloed
- * Added ability to specify more than one defualt listener for an endpoint,
- *   required by SIP which listens on both UDP and TCP.
- *
- * Revision 2.18  2004/03/13 06:25:52  rjongbloed
- * Slight rearrangement of local party name and alias list to beter match common
- *   behaviour in ancestor.
- * Abstracted local party name for endpoint into ancestor from H.,323.
- *
- * Revision 2.17  2004/03/11 06:54:27  csoutheren
- * Added ability to disable SIP or H.323 stacks
- *
- * Revision 2.16  2004/02/19 10:46:44  rjongbloed
- * Merged OpenH323 version 1.13.1 changes.
- *
- * Revision 2.15  2003/03/17 10:26:59  robertj
- * Added video support.
- *
- * Revision 2.14  2003/03/06 03:57:47  robertj
- * IVR support (work in progress) requiring large changes everywhere.
- *
- * Revision 2.13  2002/11/10 11:33:17  robertj
- * Updated to OpenH323 v1.10.3
- *
- * Revision 2.12  2002/09/16 02:52:35  robertj
- * Added #define so can select if #pragma interface/implementation is used on
- *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
- *
- * Revision 2.11  2002/07/04 07:41:47  robertj
- * Fixed memory/thread leak of transports.
- *
- * Revision 2.10  2002/07/01 04:56:30  robertj
- * Updated to OpenH323 v1.9.1
- *
- * Revision 2.9  2002/04/05 10:36:53  robertj
- * Rearranged OnRelease to remove the connection from endpoints connection
- *   list at the end of the release phase rather than the beginning.
- *
- * Revision 2.8  2002/01/22 05:04:40  robertj
- * Revamp of user input API triggered by RFC2833 support
- *
- * Revision 2.7  2001/11/14 01:31:55  robertj
- * Corrected placement of adjusting media format list.
- *
- * Revision 2.6  2001/11/13 06:25:56  robertj
- * Changed SetUpConnection() so returns BOOL as returning
- *   pointer to connection is not useful.
- *
- * Revision 2.5  2001/08/23 05:51:17  robertj
- * Completed implementation of codec reordering.
- *
- * Revision 2.4  2001/08/22 10:20:09  robertj
- * Changed connection locking to use double mutex to guarantee that
- *   no threads can ever deadlock or access deleted connection.
- *
- * Revision 2.3  2001/08/17 08:22:23  robertj
- * Moved call end reasons enum from OpalConnection to global.
- *
- * Revision 2.2  2001/08/01 05:26:35  robertj
- * Moved media formats list from endpoint to connection.
- *
- * Revision 2.1  2001/07/30 07:22:25  robertj
- * Abstracted listener management from H.323 to OpalEndPoint class.
- *
- * Revision 2.0  2001/07/27 15:48:24  robertj
- * Conversion of OpenH323 to Open Phone Abstraction Library (OPAL)
- *
+ * $Revision: 22096 $
+ * $Author: rjongbloed $
+ * $Date: 2009-02-20 21:25:23 -0600 (Fri, 20 Feb 2009) $
  */
 
-#ifndef __OPAL_ENDPOINT_H
-#define __OPAL_ENDPOINT_H
+#ifndef OPAL_OPAL_ENDPOINT_H
+#define OPAL_OPAL_ENDPOINT_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -142,11 +42,9 @@
 #include <opal/mediafmt.h>
 #include <opal/transports.h>
 
-
 class OpalCall;
 class OpalMediaStream;
 
-
 /**This class describes an endpoint base class.
    Each protocol (or psuedo-protocol) would create a descendant off this
    class to manage its particular subsystem. Typically this would involve
@@ -188,6 +86,12 @@ class OpalEndPoint : public PObject
     /**Destroy the endpoint.
      */
     ~OpalEndPoint();
+
+    /**Shut down the endpoint, this is called by the OpalManager just before
+       destroying the object and can be handy to make sure some things are
+       stopped before the vtable gets clobbered.
+      */
+    virtual void ShutDown();
   //@}
 
   /**@name Overrides from PObject */
@@ -204,24 +108,18 @@ class OpalEndPoint : public PObject
   /**@name Listeners management */
   //@{
     /**Add a set of listeners to the endoint.
-       This allows for the automatic creating of incoming call connections. An
-       application should use OnConnectionEstablished() to monitor when calls
-       have arrived and been successfully negotiated.
-
+       This allows for the automatic creating of incoming call connections. 
        If the list is empty then GetDefaultListeners() is used.
       */
-    BOOL StartListeners(
+    PBoolean StartListeners(
       const PStringArray & interfaces ///<  Address of interface to listen on.
     );
 
     /**Add a listener to the endoint.
-       This allows for the automatic creating of incoming call connections. An
-       application should use OnConnectionEstablished() to monitor when calls
-       have arrived and been successfully negotiated.
-
+       This allows for the automatic creating of incoming call connections. /
        If the address is empty then the first entry of GetDefaultListeners() is used.
       */
-    BOOL StartListener(
+    PBoolean StartListener(
       const OpalTransportAddress & iface ///<  Address of interface to listen on.
     );
 
@@ -230,7 +128,7 @@ class OpalEndPoint : public PObject
        application should use OnConnectionEstablished() to monitor when calls
        have arrived and been successfully negotiated.
       */
-    BOOL StartListener(
+    PBoolean StartListener(
       OpalListener * listener ///<  Transport dependent listener.
     );
 
@@ -240,18 +138,31 @@ class OpalEndPoint : public PObject
       */
     virtual PStringArray GetDefaultListeners() const;
 
+    /**Find a listener given the transport address.
+      */
+    OpalListener * FindListener(
+        const OpalTransportAddress & iface ///<  Address of interface we may be listening on.
+    );
+
+    /**Stop a listener given the transport address.
+       Returns PTrue if a listener was on that interface, and was stopped.
+      */
+    PBoolean StopListener(
+        const OpalTransportAddress & iface ///<  Address of interface we may be listening on.
+    );
+
     /**Remove a listener from the endoint.
        If the listener parameter is NULL then all listeners are removed.
       */
-    BOOL RemoveListener(
+    PBoolean RemoveListener(
       OpalListener * listener ///<  Transport dependent listener.
     );
 
     /**Return a list of the transport addresses for all listeners on this endpoint
       */
     OpalTransportAddressArray GetInterfaceAddresses(
-      BOOL excludeLocalHost = TRUE,       ///<  Flag to exclude 127.0.0.1
-      OpalTransport * associatedTransport = NULL
+      PBoolean excludeLocalHost = PTrue,       ///<  Flag to exclude 127.0.0.1
+      const OpalTransport * associatedTransport = NULL
                           ///<  Associated transport for precedence and translation
     );
 
@@ -263,13 +174,13 @@ class OpalEndPoint : public PObject
 
     /**Handle new incoming connection from listener.
 
-       A return value of TRUE indicates that the transport object should be
-       deleted by the caller. FALSE indicates that something else (eg the
+       A return value of PTrue indicates that the transport object should be
+       deleted by the caller. PFalse indicates that something else (eg the
        connection) has taken over responsibility for deleting the transport.
 
-       The default behaviour just returns TRUE.
+       The default behaviour just returns PTrue.
       */
-    virtual BOOL NewIncomingConnection(
+    virtual PBoolean NewIncomingConnection(
       OpalTransport * transport  ///<  Transport connection came in on
     );
   //@}
@@ -292,12 +203,12 @@ class OpalEndPoint : public PObject
 
        The proto field is optional when passed to a specific endpoint. If it
        is present, however, it must agree with the endpoints protocol name or
-       FALSE is returned.
+       PFalse is returned.
 
        This function usually returns almost immediately with the connection
        continuing to occur in a new background thread.
 
-       If FALSE is returned then the connection could not be established. For
+       If PFalse is returned then the connection could not be established. For
        example if a PSTN endpoint is used and the assiciated line is engaged
        then it may return immediately. Returning a non-NULL value does not
        mean that the connection will succeed, only that an attempt is being
@@ -305,17 +216,24 @@ class OpalEndPoint : public PObject
 
        The default behaviour is pure.
      */
-    virtual BOOL MakeConnection(
-      OpalCall & call,        ///<  Owner of connection
-      const PString & party,  ///<  Remote party to call
-      void * userData         ///<  Arbitrary data to pass to connection
+    virtual PBoolean MakeConnection(
+      OpalCall & call,          ///<  Owner of connection
+      const PString & party,    ///<  Remote party to call
+      void * userData = NULL,          ///<  Arbitrary data to pass to connection
+      unsigned int options = 0,     ///<  options to pass to conneciton
+      OpalConnection::StringOptions * stringOptions = NULL
     ) = 0;
 
+    /**Callback for outgoing connection, it is invoked after OpalLineConnection::SetUpConnection
+       This function allows the application to set up some parameters or to log some messages
+       */
+    virtual PBoolean OnSetUpConnection(OpalConnection &connection);
+    
     /**Call back for answering an incoming call.
        This function is used for an application to control the answering of
        incoming calls.
 
-       If TRUE is returned then the connection continues. If FALSE then the
+       If PTrue is returned then the connection continues. If PFalse then the
        connection is aborted.
 
        Note this function should not block for any length of time. If the
@@ -332,16 +250,43 @@ class OpalEndPoint : public PObject
 
        The default behaviour calls the OpalManager function of the same name.
      */
-    virtual BOOL OnIncomingConnection(
+    virtual PBoolean OnIncomingConnection(
+      OpalConnection & connection,  ///<  Connection that is calling
+      unsigned options,             ///<  options for new connection (can't use default value as overrides will fail)
+      OpalConnection::StringOptions * stringOptions
+    );
+    virtual PBoolean OnIncomingConnection(
+      OpalConnection & connection,  ///<  Connection that is calling
+      unsigned options              ///<  options for new connection (can't use default value as overrides will fail)
+    );
+    virtual PBoolean OnIncomingConnection(
       OpalConnection & connection   ///<  Connection that is calling
     );
 
+    /**Call back for remote party is now responsible for completing the call.
+       This function is called when the remote system has been contacted and it
+       has accepted responsibility for completing, or failing, the call. This
+       is distinct from OnAlerting() in that it is not known at this time if
+       anything is ringing. This indication may be used to distinguish between
+       "transport" level error, in which case another host may be tried, and
+       that finalising the call has moved "upstream" and the local system has
+       no more to do but await a result.
+
+       If an application overrides this function, it should generally call the
+       ancestor version for correct operation.
+
+       The default behaviour calls the OpalManager function of the same name.
+     */
+    virtual void OnProceeding(
+      OpalConnection & connection   ///<  Connection that is proceeeding
+    );
+
     /**Call back for remote party being alerted.
        This function is called after the connection is informed that the
        remote endpoint is "ringing". Generally some time after the
        MakeConnection() function was called, this is function is called.
 
-       If FALSE is returned the connection is aborted.
+       If PFalse is returned the connection is aborted.
 
        If an application overrides this function, it should generally call the
        ancestor version for correct operation. An application would typically
@@ -352,7 +297,7 @@ class OpalEndPoint : public PObject
        The default behaviour calls the OpalManager function of the same name.
      */
     virtual void OnAlerting(
-      OpalConnection & connection   ///<  Connection that was established
+      OpalConnection & connection   ///<  Connection that is alerting
     );
 
     /**Call back for answering an incoming call.
@@ -391,9 +336,9 @@ class OpalEndPoint : public PObject
     );
 
     /**A call back function whenever a connection is established.
-       This indicates that a connection to an endpoint was established. That
-       is the endpoint received acknowledgement via whatever protocol it uses
-       that the connection may now start media streams.
+       This indicates that a connection to an endpoint was established. This
+       usually occurs after OnConnected() and indicates that the connection
+       is both connected and has media flowing.
 
        In the context of H.323 this means that the signalling and control
        channels are open and the TerminalCapabilitySet and MasterSlave
@@ -433,15 +378,18 @@ class OpalEndPoint : public PObject
 
        The default behaviour calls the OpalManager function of the same name.
       */
-    void OnHold(
-      OpalConnection & connection   ///<  Connection that was held
+    virtual void OnHold(
+      OpalConnection & connection,   ///<  Connection that was held/retrieved
+      bool fromRemote,               ///<  Indicates remote has held local connection
+      bool onHold                    ///<  Indicates have just been held/retrieved.
     );
+    virtual void OnHold(OpalConnection & connection); // For backward compatibility
 
     /**A call back function whenever a connection is forwarded.
 
        The default behaviour does nothing.
       */
-    virtual BOOL OnForwarded(
+    virtual PBoolean OnForwarded(
       OpalConnection & connection,  ///<  Connection that was held
       const PString & remoteParty         ///<  The new remote party
     );
@@ -453,7 +401,7 @@ class OpalEndPoint : public PObject
        disposal happens at some later time by a background thread. This it is
        safe to call this function from anywhere.
       */
-    virtual BOOL ClearCall(
+    virtual PBoolean ClearCall(
       const PString & token,    ///<  Token for identifying connection
       OpalConnection::CallEndReason reason = OpalConnection::EndedByLocalUser, ///<  Reason for call clearing
       PSyncPoint * sync = NULL  ///<  Sync point to wait on.
@@ -463,7 +411,7 @@ class OpalEndPoint : public PObject
        This hangs up the connection to a remote endpoint. Note that these functions
        are synchronous
       */
-    virtual BOOL ClearCallSynchronous(
+    virtual PBoolean ClearCallSynchronous(
       const PString & token,    ///<  Token for identifying connection
       OpalConnection::CallEndReason reason = OpalConnection::EndedByLocalUser, ///<  Reason for call clearing
       PSyncPoint * sync = NULL  ///<  Sync point to wait on.
@@ -477,7 +425,7 @@ class OpalEndPoint : public PObject
       */
     virtual void ClearAllCalls(
       OpalConnection::CallEndReason reason = OpalConnection::EndedByLocalUser, ///<  Reason for call clearing
-      BOOL wait = TRUE   ///<  Flag for wait for calls to e cleared.
+      PBoolean wait = PTrue   ///<  Flag for wait for calls to e cleared.
     );
 
     /**Find a connection that uses the specified token.
@@ -489,13 +437,41 @@ class OpalEndPoint : public PObject
       PSafetyMode mode = PSafeReadWrite
     ) { return connectionsActive.FindWithLock(token, mode); }
 
+    /**Find a connection that uses the specified token.
+       This searches the endpoint for the connection that contains the token
+       as provided by functions such as MakeConnection(). If not then it
+       attempts to use the token as a OpalCall token and find a connection
+       of the same class.
+      */
+    template <class ConnClass>
+    PSafePtr<ConnClass> GetConnectionWithLockAs(
+      const PString & token,     ///<  Token to identify connection
+      PSafetyMode mode = PSafeReadWrite
+    )
+    {
+      PSafePtr<ConnClass> connection = PSafePtrCast<OpalConnection, ConnClass>(GetConnectionWithLock(token, mode));
+      if (connection == NULL) {
+        PSafePtr<OpalCall> call = manager.FindCallWithLock(token, PSafeReadOnly);
+        if (call != NULL) {
+          connection = PSafePtrCast<OpalConnection, ConnClass>(call->GetConnection(0, mode));
+          if (connection == NULL)
+            connection = PSafePtrCast<OpalConnection, ConnClass>(call->GetConnection(1, mode));
+        }
+      }
+      return connection;
+    }
+
     /**Get all calls current on the endpoint.
       */
     PStringList GetAllConnections();
+    
+    /** Get calls count on the endpoint
+      */
+    PINDEX GetConnectionCount() const { return connectionsActive.GetSize(); }
 
     /**Determine if a connection is active.
       */
-    virtual BOOL HasConnection(
+    virtual PBoolean HasConnection(
       const PString & token   ///<  Token for identifying connection
     );
 
@@ -515,9 +491,10 @@ class OpalEndPoint : public PObject
        Note that a specific connection may not actually support all of the
        media formats returned here, but should return no more.
 
-       The default behaviour is pure.
+       The default behaviour returns the most basic media formats, PCM audio
+       and YUV420P video.
       */
-    virtual OpalMediaFormatList GetMediaFormats() const = 0;
+    virtual OpalMediaFormatList GetMediaFormats() const;
 
     /**Adjust media formats available on a connection.
        This is called by a connection after it has called
@@ -543,7 +520,7 @@ class OpalEndPoint : public PObject
 
        The default behaviour calls the OpalManager function of the same name.
       */
-    virtual BOOL OnOpenMediaStream(
+    virtual PBoolean OnOpenMediaStream(
       OpalConnection & connection,  ///<  Connection that owns the media stream
       OpalMediaStream & stream      ///<  New media stream being opened
     );
@@ -556,6 +533,7 @@ class OpalEndPoint : public PObject
       const OpalMediaStream & stream     ///<  Media stream being closed
     );
 
+#if OPAL_VIDEO
     /**Add video media formats available on a connection.
 
        The default behaviour calls the OpalEndPoint function of the same name.
@@ -567,23 +545,24 @@ class OpalEndPoint : public PObject
 
     /**Create an PVideoInputDevice for a source media stream.
       */
-    virtual BOOL CreateVideoInputDevice(
+    virtual PBoolean CreateVideoInputDevice(
       const OpalConnection & connection,    ///<  Connection needing created video device
       const OpalMediaFormat & mediaFormat,  ///<  Media format for stream
       PVideoInputDevice * & device,         ///<  Created device
-      BOOL & autoDelete                     ///<  Flag for auto delete device
+      PBoolean & autoDelete                     ///<  Flag for auto delete device
     );
 
     /**Create an PVideoOutputDevice for a sink media stream or the preview
        display for a source media stream.
       */
-    virtual BOOL CreateVideoOutputDevice(
+    virtual PBoolean CreateVideoOutputDevice(
       const OpalConnection & connection,    ///<  Connection needing created video device
       const OpalMediaFormat & mediaFormat,  ///<  Media format for stream
-      BOOL preview,                         ///<  Flag indicating is a preview output
+      PBoolean preview,                         ///<  Flag indicating is a preview output
       PVideoOutputDevice * & device,        ///<  Created device
-      BOOL & autoDelete                     ///<  Flag for auto delete device
+      PBoolean & autoDelete                     ///<  Flag for auto delete device
     );
+#endif
   //@}
 
   /**@name User indications */
@@ -608,38 +587,32 @@ class OpalEndPoint : public PObject
       char tone,                    ///<  Tone received
       int duration                  ///<  Duration of tone
     );
+
+    /**Read a sequence of user indications from connection with timeouts.
+      */
+    virtual PString ReadUserInput(
+      OpalConnection & connection,        ///<  Connection to read input from
+      const char * terminators = "#\r\n", ///<  Characters that can terminte input
+      unsigned lastDigitTimeout = 4,      ///<  Timeout on last digit in string
+      unsigned firstDigitTimeout = 30     ///<  Timeout on receiving any digits
+    );
   //@}
 
   /**@name Other services */
   //@{
-    /**Create an instance of the T.120 protocol handler.
-       This is called when the OpenLogicalChannel subsystem requires that
-       a T.120 channel be established.
+    /**Callback called when Message Waiting Indication (MWI) is received
+     */
+    virtual void OnMWIReceived (
+      const PString & party,                ///< Name of party MWI is for
+      OpalManager::MessageWaitingType type, ///< Type of message that is waiting
+      const PString & extraInfo             ///< Addition information on the MWI
+    );
 
-       Note that if the application overrides this it should return a pointer to a
-       heap variable (using new) as it will be automatically deleted when the
-       H323Connection is deleted.
-
-       The default behavour calls the OpalManager function of the same name.
+    /** Execute garbage collection for endpoint.
+        Returns PTrue if all garbage has been collected.
+        Default behaviour deletes the objects in the connectionsActive list.
       */
-    virtual OpalT120Protocol * CreateT120ProtocolHandler(
-      const OpalConnection & connection  ///<  Connection for which T.120 handler created
-    ) const;
-
-    /**Create an instance of the T.38 protocol handler.
-       This is called when the OpenLogicalChannel subsystem requires that
-       a T.38 fax channel be established.
-
-       Note that if the application overrides this it should return a pointer to a
-       heap variable (using new) as it will be automatically deleted when the
-       H323Connection is deleted.
-
-       The default behavour calls the OpalManager function of the same name.
-      */
-    virtual OpalT38Protocol * CreateT38ProtocolHandler(
-      const OpalConnection & connection  ///<  Connection for which T.38 handler created
-    ) const;
-
+    virtual PBoolean GarbageCollection();
   //@}
 
   /**@name Member variable access */
@@ -654,19 +627,31 @@ class OpalEndPoint : public PObject
 
     /**Get an indication of if this endpoint has particular option.
       */
-    BOOL HasAttribute(Attributes opt) const { return (attributeBits&opt) != 0; }
+    PBoolean HasAttribute(Attributes opt) const { return (attributeBits&opt) != 0; }
 
-    /**Get the initial bandwidth parameter.
+    /**Get the default signal port for this endpoint.
      */
     WORD GetDefaultSignalPort() const { return defaultSignalPort; }
 
+    /**Get the product info for all endpoints.
+      */
+    const OpalProductInfo & GetProductInfo() const { return productInfo; }
+
+    /**Set the product info for all endpoints.
+      */
+    void SetProductInfo(
+      const OpalProductInfo & info
+    ) { productInfo = info; }
+
     /**Get the default local party name for all connections on this endpoint.
       */
     const PString & GetDefaultLocalPartyName() const { return defaultLocalPartyName; }
 
     /**Set the default local party name for all connections on this endpoint.
       */
-    void SetDefaultLocalPartyName(const PString & name) { defaultLocalPartyName = name; }
+    virtual void SetDefaultLocalPartyName(
+      const PString & name  /// Name for local party
+    ) { defaultLocalPartyName = name; }
 
     /**Get the default local display name for all connections on this endpoint.
       */
@@ -687,35 +672,85 @@ class OpalEndPoint : public PObject
     /**Get the set of listeners (incoming call transports) for this endpoint.
      */
     const OpalListenerList & GetListeners() const { return listeners; }
+
+    /**Get the default options for created connections.
+      */
+    const OpalConnection::StringOptions & GetDefaultStringOptions() const { return m_defaultStringOptions; }
+
+    /**Set the default options for created connections.
+      */
+    void SetDefaultStringOptions(const OpalConnection::StringOptions & opts) { m_defaultStringOptions = opts; }
+
+    /**Set the default option for created connections.
+      */
+    void SetDefaultStringOption(const PCaselessString & key, const PString & data) { m_defaultStringOptions.SetAt(key, data); }
+
+    /**Get the default mode for sending User Input Indications.
+      */
+    OpalConnection::SendUserInputModes GetSendUserInputMode() const { return defaultSendUserInputMode; }
+
+    /**Set the default mode for sending User Input Indications.
+      */
+    void SetSendUserInputMode(OpalConnection::SendUserInputModes mode) { defaultSendUserInputMode = mode; }
   //@}
 
+    virtual PString GetDefaultTransport() const;
+
+    /**Call back for a new connection has been constructed.
+       This is called after CreateConnection has returned a new connection.
+       It allows an application to make any custom adjustments to the
+       connection before it begins to process the protocol. behind it.
+      */
+    virtual void OnNewConnection(
+      OpalCall & call,              ///< Call that owns the newly created connection.
+      OpalConnection & connection   ///< New connection just created
+    );
+
+#if OPAL_PTLIB_SSL
+    PString GetSSLCertificate() const;
+#endif
+
+    /** Find a listener that is compatible with the specified protocol
+     */
+    bool FindListenerForProtocol(const char * protoPrefix, OpalTransportAddress & addr);
+
+    /**Add IM media formats available on a connection.
+       The default behaviour calls the OpalEndPoint function of the same name.
+      */
+    virtual void AddIMMediaFormats(
+      OpalMediaFormatList & mediaFormats, ///<  Media formats to use
+      const OpalConnection * connection = NULL  ///<  Optional connection that is using formats
+    ) const;
 
   protected:
     OpalManager   & manager;
     PCaselessString prefixName;
     unsigned        attributeBits;
     WORD            defaultSignalPort;
+    OpalProductInfo productInfo;
     PString         defaultLocalPartyName;
     PString         defaultDisplayName;
 
     unsigned initialBandwidth;  // in 100s of bits/sev
+    OpalConnection::StringOptions      m_defaultStringOptions;
+    OpalConnection::SendUserInputModes defaultSendUserInputMode;
 
     OpalListenerList   listeners;
-    PSyncPoint         allConnectionsCleared;
 
     class ConnectionDict : public PSafeDictionary<PString, OpalConnection>
     {
         virtual void DeleteObject(PObject * object) const;
     } connectionsActive;
+    PBoolean AddConnection(OpalConnection * connection);
 
     PMutex inUseFlag;
 
-  friend void OpalManager::GarbageCollection();
-  friend void OpalConnection::Release(CallEndReason reason);
+    friend void OpalManager::GarbageCollection();
+    friend void OpalConnection::Release(CallEndReason reason);
 };
 
 
-#endif // __OPAL_ENDPOINT_H
+#endif // OPAL_OPAL_ENDPOINT_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
