@@ -24,164 +24,31 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: sippdu.h,v $
- * Revision 2.36.2.5  2007/01/15 22:16:43  dsandras
- * Backported patches improving stability from HEAD to Phobos.
- *
- * Revision 2.36.2.4  2006/12/08 06:27:20  csoutheren
- * Fix compilation problem caused by bad patch backports
- * Allow compilation with latest PWLib
- *
- * Revision 2.36.2.3  2006/08/07 20:47:37  dsandras
- * Backported fix from HEAD.
- *
- * Revision 2.36.2.2  2006/08/07 20:08:54  dsandras
- * Backported qop support from HEAD.
- *
- * Revision 2.36.2.1  2006/08/07 19:54:36  dsandras
- * Backported fix from HEAD to add support for the opaque attribute when
- * authenticating.
- *
- * Revision 2.36  2006/01/02 11:28:07  dsandras
- * Some documentation. Various code cleanups to prevent duplicate code.
- *
- * Revision 2.35  2005/12/04 15:02:00  dsandras
- * Fixed IP translation in the VIA field of most request PDUs.
- *
- * Revision 2.34  2005/11/30 13:35:26  csoutheren
- * Changed tags for Doxygen
- *
- * Revision 2.33  2005/11/07 06:34:53  csoutheren
- * Changed PMutex to PTimedMutex
- *
- * Revision 2.32  2005/10/22 17:14:45  dsandras
- * Send an OPTIONS request periodically when STUN is being used to maintain the registrations binding alive.
- *
- * Revision 2.31  2005/09/27 16:06:12  dsandras
- * Added function that returns the address to which a request should be sent
- * according to the RFC.
- * Removed OnCompleted method for SIPInvite, the ACK is now
- * sent from the SIPConnection class so that the response has been processed
- * for Record Route headers.
- * Added class for the ACK request to make the distinction between an ACK sent
- * for a 2xx response (in a dialog request) and an ACK sent for a non-2xx response.
- *
- * Revision 2.30  2005/09/21 19:49:25  dsandras
- * Added a function that returns the transport address where to send responses to incoming requests according to RFC3261 and RFC3581.
- *
- * Revision 2.29  2005/09/20 16:59:32  dsandras
- * Added method that adjusts the VIA field of incoming requests accordingly to the SIP RFC and RFC 3581 if the transport address/port do not correspond to what is specified in the Via. Thanks Ted Szoczei for the feedback.
- *
- * Revision 2.28  2005/08/10 19:34:34  dsandras
- * Added helper functions to get and set values of parameters in PDU fields.
- *
- * Revision 2.27  2005/06/04 12:44:36  dsandras
- * Applied patch from Ted Szoczei to fix leaks and problems on cancelling a call and to improve the Allow PDU field handling.
- *
- * Revision 2.26  2005/05/06 07:37:06  csoutheren
- * Various changed while working with SIP carrier
- *   - remove assumption that authentication realm is a domain name.
- *   - stopped rewrite of "To" field when proxy being used
- *   - fix Contact field in REGISTER to match actual port used when Symmetric NATin use
- *   - lots of formatting changes and cleanups
- *
- * Revision 2.25  2005/04/28 20:22:54  dsandras
- * Applied big sanity patch for SIP thanks to Ted Szoczei <tszoczei@microtronix.ca>.
- * Thanks a lot!
- *
- * Revision 2.24  2005/04/28 07:59:37  dsandras
- * Applied patch from Ted Szoczei to fix problem when answering to PDUs containing
- * multiple Via fields in the message header. Thanks!
- *
- * Revision 2.23  2005/04/11 11:12:38  dsandras
- * Added Method_MESSAGE support for future use.
- *
- * Revision 2.22  2005/04/10 21:18:24  dsandras
- * Added support for the SIPMessage PDU.
- *
- * Revision 2.21  2005/04/10 21:05:14  dsandras
- * Added support for SIP Invite using the same RTP Session (call hold).
- *
- * Revision 2.20  2005/04/10 21:04:08  dsandras
- * Added support for Blind Transfer (SIP REFER).
- *
- * Revision 2.19  2005/03/11 18:12:08  dsandras
- * Added support to specify the realm when registering. That way softphones already know what authentication information to use when required. The realm/domain can also be used in the From field.
- *
- * Revision 2.18  2005/02/19 22:48:48  dsandras
- * Added the possibility to register to several registrars and be able to do authenticated calls to each of them. Added SUBSCRIBE/NOTIFY support for Message Waiting Indications.
- *
- * Revision 2.17  2004/12/12 12:31:03  dsandras
- * GetDisplayName now contains more complex code.
- *
- * Revision 2.16  2004/08/22 12:27:44  rjongbloed
- * More work on SIP registration, time to live refresh and deregistration on exit.
- *
- * Revision 2.15  2004/03/14 10:14:13  rjongbloed
- * Changes to REGISTER to support authentication
- *
- * Revision 2.14  2004/03/14 08:34:09  csoutheren
- * Added ability to set User-Agent string
- *
- * Revision 2.13  2004/03/13 06:32:17  rjongbloed
- * Fixes for removal of SIP and H.323 subsystems.
- * More registration work.
- *
- * Revision 2.12  2004/03/09 12:09:55  rjongbloed
- * More work on SIP register.
- *
- * Revision 2.11  2003/12/16 10:22:45  rjongbloed
- * Applied enhancements, thank you very much Ted Szoczei
- *
- * Revision 2.10  2003/12/15 11:56:17  rjongbloed
- * Applied numerous bug fixes, thank you very much Ted Szoczei
- *
- * Revision 2.9  2002/09/16 02:52:35  robertj
- * Added #define so can select if #pragma interface/implementation is used on
- *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
- *
- * Revision 2.8  2002/04/16 07:53:15  robertj
- * Changes to support calls through proxies.
- *
- * Revision 2.7  2002/04/12 12:23:03  robertj
- * Allowed for endpoint listener that is not on port 5060.
- *
- * Revision 2.6  2002/04/10 08:12:17  robertj
- * Added call back for when transaction completed, used for invite descendant.
- *
- * Revision 2.5  2002/04/10 03:16:02  robertj
- * Major changes to RTP session management when initiating an INVITE.
- * Improvements in error handling and transaction cancelling.
- *
- * Revision 2.4  2002/04/09 01:02:14  robertj
- * Fixed problems with restarting INVITE on  authentication required response.
- *
- * Revision 2.3  2002/04/05 10:42:04  robertj
- * Major changes to support transactions (UDP timeouts and retries).
- *
- * Revision 2.2  2002/03/08 06:28:19  craigs
- * Changed to allow Authorisation to be included in other PDUs
- *
- * Revision 2.1  2002/02/01 04:53:01  robertj
- * Added (very primitive!) SIP support.
- *
+ * $Revision: 23493 $
+ * $Author: rjongbloed $
+ * $Date: 2009-09-18 21:00:38 -0500 (Fri, 18 Sep 2009) $
  */
 
-#ifndef __OPAL_SIPPDU_H
-#define __OPAL_SIPPDU_H
+#ifndef OPAL_SIP_SIPPDU_H
+#define OPAL_SIP_SIPPDU_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
+#include <opal/buildopts.h>
+
+#if OPAL_SIP
 
 #include <ptclib/mime.h>
 #include <ptclib/url.h>
 #include <sip/sdp.h>
+#include <opal/rtpconn.h>
 
-
+ 
 class OpalTransport;
 class OpalTransportAddress;
+class OpalProductInfo;
 
 class SIPEndPoint;
 class SIPConnection;
@@ -229,6 +96,23 @@ class SIPURL : public PURL
       WORD listenerPort = 0
     );
 
+    SIPURL(
+      const OpalTransportAddress & _address, 
+      WORD listenerPort = 0
+    );
+
+    /**Compare the two SIPURLs and return their relative rank.
+       Note that does an intelligent comparison according to the rules
+       in RFC3261 Section 19.1.4.
+
+     @return
+       #LessThan#, #EqualTo# or #GreaterThan#
+       according to the relative rank of the objects.
+     */
+    virtual Comparison Compare(
+      const PObject & obj   ///< Object to compare against.
+    ) const;
+
     /** Returns complete SIPURL as one string, including displayname (in
         quotes) and address in angle brackets.
       */
@@ -236,19 +120,62 @@ class SIPURL : public PURL
 
     /** Returns display name only
       */
-    PString GetDisplayName() const;
+    PString GetDisplayName(PBoolean useDefault = PTrue) const;
     
     void SetDisplayName(const PString & str) 
       { displayName = str; }
-    
+
+    /**Returns the field parameter (outside of <>)
+      */
+    PString GetFieldParameters() const { return fieldParameters; }
+
+    /**Returns the field parameter (outside of <>)
+      */
+    void SetFieldParameters(const PString & str ) { fieldParameters = str; }
+
+    /**Get the host and port as a transport address.
+      */
     OpalTransportAddress GetHostAddress() const;
 
+    enum UsageContext {
+      ExternalURI,  ///< URI used anywhere outside of protocol
+      RequestURI,   ///< Request-URI (after the INVITE)
+      ToURI,        ///< To header field
+      FromURI,      ///< From header field
+      ContactURI,   ///< Registration or Redirection Contact header field
+      RouteURI,     ///< Dialog Contact header field, or Record-Route header field
+      RegisterURI   ///< URI on REGISTER request line.
+    };
+
     /** Removes tag parm & query vars and recalculates urlString
         (scheme, user, password, host, port & URI parms (like transport))
+        which are not allowed in the context specified, e.g. Request-URI etc
+        According to RFC3261, 19.1.1 Table 1
       */
-    void AdjustForRequestURI();
+    void Sanitise(
+      UsageContext context  ///< Context for URI
+    );
+
+    /** This will adjust the current URL according to RFC3263, using DNS SRV records.
+
+        @return FALSE if DNS is available but entry is larger than last SRV record entry,
+                TRUE if DNS lookup fails or no DNS is available
+      */
+    PBoolean AdjustToDNS(
+      PINDEX entry = 0  /// Entry in the SRV record to adjust to
+    );
+
+    /// Generate a unique string suitable as a dialog tag
+    static PString GenerateTag();
+
+    /// Set a tag with a new unique ID.
+    void SetTag(
+      const PString & tag = GenerateTag()
+    );
 
   protected:
+    void ParseAsAddress(const PString & name, const OpalTransportAddress & _address, WORD listenerPort = 0);
+
     /** Parses name-addr, like:
         "displayname"<scheme:user:password@host:port;transport=type>;tag=value
         into:
@@ -265,14 +192,15 @@ class SIPURL : public PURL
         fragment
 
         Note that tag parameter outside of <> will be lost,
-        but tag in URL without <> will be kept until AdjustForRequestURI
+        but tag in URL without <> will be kept until Sanitise()
      */
-    virtual BOOL InternalParse(
+    virtual PBoolean InternalParse(
       const char * cstr,
       const char * defaultScheme
     );
 
     PString displayName;
+    PString fieldParameters;
 };
 
 
@@ -313,19 +241,28 @@ class SIPMIMEInfo : public PMIMEInfo
 {
   PCLASSINFO(SIPMIMEInfo, PMIMEInfo);
   public:
-    SIPMIMEInfo(BOOL compactForm = FALSE);
+    SIPMIMEInfo(bool compactForm = false);
 
-    void SetForm(BOOL v) { compactForm = v; }
+    virtual void PrintOn(ostream & strm) const;
+    virtual void ReadFrom(istream & strm);
 
-    PString GetContentType() const;
+    void SetCompactForm(bool form) { compactForm = form; }
+
+    PCaselessString GetContentType(bool includeParameters = false) const;
     void SetContentType(const PString & v);
 
-    PString GetContentEncoding() const;
+    PCaselessString GetContentEncoding() const;
     void SetContentEncoding(const PString & v);
 
     PString GetFrom() const;
     void SetFrom(const PString & v);
 
+    PString GetPAssertedIdentity() const;
+    void SetPAssertedIdentity(const PString & v);
+
+    PString GetPPreferredIdentity() const;
+    void SetPPreferredIdentity(const PString & v);
+
     PString GetAccept() const;
     void SetAccept(const PString & v);
 
@@ -342,6 +279,7 @@ class SIPMIMEInfo : public PMIMEInfo
     void SetCallID(const PString & v);
 
     PString GetContact() const;
+    bool GetContacts(std::list<SIPURL> & contacts) const;
     void SetContact(const PString & v);
     void SetContact(const SIPURL & url);
 
@@ -365,7 +303,7 @@ class SIPMIMEInfo : public PMIMEInfo
 
     PINDEX  GetContentLength() const;
     void SetContentLength(PINDEX v);
-		BOOL IsContentLengthPresent() const;
+    PBoolean IsContentLengthPresent() const;
 
     PString GetCSeq() const;
     void SetCSeq(const PString & v);
@@ -388,9 +326,10 @@ class SIPMIMEInfo : public PMIMEInfo
     void SetProxyAuthenticate(const PString & v);
 
     PStringList GetRoute() const;
+    void SetRoute(const PString & v);
     void SetRoute(const PStringList & v);
 
-    PStringList GetRecordRoute() const;
+    PStringList GetRecordRoute(bool reversed) const;
     void SetRecordRoute(const PStringList & v);
 
     unsigned GetCSeqIndex() const { return GetCSeq().AsUnsigned(); }
@@ -404,50 +343,81 @@ class SIPMIMEInfo : public PMIMEInfo
     PString GetEvent() const;
     void SetEvent(const PString & v);
     
-    PString GetSubscriptionState() const;
+    PCaselessString GetSubscriptionState() const;
     void SetSubscriptionState(const PString & v);
     
     PString GetUserAgent() const;
-    void SetUserAgent(const SIPEndPoint & sipep);        // normally "OPAL/2.0"
+    void SetUserAgent(const PString & v);
+
+    PString GetOrganization() const;
+    void SetOrganization(const PString & v);
+
+    void GetProductInfo(OpalProductInfo & info) const;
+    void SetProductInfo(const PString & ua, const OpalProductInfo & info);
 
     PString GetWWWAuthenticate() const;
     void SetWWWAuthenticate(const PString & v);
 
+    PString GetSIPIfMatch() const;
+    void SetSIPIfMatch(const PString & v);
+
+    PString GetSIPETag() const;
+    void SetSIPETag(const PString & v);
+
+    PString GetRequire() const;
+    void SetRequire(const PString & v, bool overwrite);
+
+    void GetAlertInfo(PString & info, int & appearance);
+    void SetAlertInfo(const PString & info, int appearance);
 
     /** return the value of a header field parameter, empty if none
      */
-    PString GetFieldParameter(const PString &,
-			      const PString &);
-    
+    PString GetFieldParameter(
+      const PString & fieldName,    ///< Field name in dictionary
+      const PString & paramName,    ///< Field parameter name
+      const PString & defaultValue = PString::Empty()  ///< Default value for parameter
+    ) const { return ExtractFieldParameter((*this)(fieldName), paramName, defaultValue); }
+
     /** set the value for a header field parameter, replace the
      *  current value, or add the parameter and its
      *  value if not already present.
      */
-    void SetFieldParameter(const PString &,
-			   PString &,
-			   const PString &);
-    
-    /** return TRUE if the header field parameter is present
+    void SetFieldParameter(
+      const PString & fieldName,    ///< Field name in dictionary
+      const PString & paramName,    ///< Field parameter name
+      const PString & newValue      ///< New value for parameter
+    ) { SetAt(fieldName, InsertFieldParameter((*this)(fieldName), paramName, newValue)); }
+
+    /** return the value of a header field parameter, empty if none
      */
-    BOOL HasFieldParameter(const PString &,
-			   const PString &);
+    static PString ExtractFieldParameter(
+      const PString & fieldValue,   ///< Value of field string
+      const PString & paramName,    ///< Field parameter name
+      const PString & defaultValue = PString::Empty()  ///< Default value for parameter
+    );
+
+    /** set the value for a header field parameter, replace the
+     *  current value, or add the parameter and its
+     *  value if not already present.
+     */
+    static PString InsertFieldParameter(
+      const PString & fieldValue,   ///< Value of field string
+      const PString & paramName,    ///< Field parameter name
+      const PString & newValue      ///< New value for parameter
+    );
 
   protected:
     	/** return list of route values from internal comma-delimited list
 	 */
-    PStringList GetRouteList(const char * name) const;
+    PStringList GetRouteList(const char * name, bool reversed) const;
 
 	/** store string list as one comma-delimited string of route values
 	    value formed as "<v[0]>,<v[1]>,<v[2]>" etc
 	 */
     void SetRouteList(const char * name, const PStringList & v);
 
-	/** return string keyed by full or compact header
-	 */
-    PString GetFullOrCompact(const char * fullForm, char compactForm) const;
-
     /// Encode using compact form
-    BOOL compactForm;
+    bool compactForm;
 };
 
 
@@ -458,75 +428,96 @@ class SIPAuthentication : public PObject
 {
   PCLASSINFO(SIPAuthentication, PObject);
   public:
-    SIPAuthentication(
-      const PString & username = PString::Empty(),
-      const PString & password = PString::Empty()
+    SIPAuthentication();
+
+    virtual Comparison Compare(
+      const PObject & other
+    ) const;
+
+    virtual PBoolean Parse(
+      const PString & auth,
+      PBoolean proxy
+    ) = 0;
+
+    virtual PBoolean Authorise(
+      SIP_PDU & pdu
+    ) const =  0;
+
+    virtual PBoolean IsProxy() const               { return isProxy; }
+
+    virtual PString GetUsername() const   { return username; }
+    virtual PString GetPassword() const   { return password; }
+    virtual PString GetAuthRealm() const  { return PString::Empty(); }
+
+    virtual void SetUsername(const PString & user) { username = user; }
+    virtual void SetPassword(const PString & pass) { password = pass; }
+    virtual void SetAuthRealm(const PString &)     { }
+
+    PString GetAuthParam(const PString & auth, const char * name) const;
+    PString AsHex(PMessageDigest5::Code & digest) const;
+    PString AsHex(const PBYTEArray & data) const;
+
+    static SIPAuthentication * ParseAuthenticationRequired(bool isProxy,
+                                                const PString & line,
+                                                      PString & errorMsg);
+
+  protected:
+    PBoolean  isProxy;
+
+    PString   username;
+    PString   password;
+};
+
+typedef PFactory<SIPAuthentication> SIPAuthenticationFactory;
+
+/////////////////////////////////////////////////////////////////////////
+
+class SIPDigestAuthentication : public SIPAuthentication
+{
+  PCLASSINFO(SIPDigestAuthentication, SIPAuthentication);
+  public:
+    SIPDigestAuthentication();
+
+    SIPDigestAuthentication & operator =(
+      const SIPDigestAuthentication & auth
     );
 
-#if PWLIB_MAJOR >= 1 && PWLIB_MINOR >= 11
-    SIPAuthentication & operator =(const SIPAuthentication & auth)
-    {
-      isProxy   = auth.isProxy;
-      authRealm = auth.authRealm;
-      username  = auth.username;
-      password  = auth.password;
-      nonce     = auth.nonce;
-      algorithm = auth.algorithm;
-		  opaque    = auth.opaque;
-              
-		  qopAuth    = auth.qopAuth;
-		  qopAuthInt = auth.qopAuthInt;
-		  cnonce     = auth.cnonce;
-		  nonceCount.SetValue(auth.nonceCount);
+    virtual Comparison Compare(
+      const PObject & other
+    ) const;
 
-      return *this;
-    }
-#endif
-
-    BOOL Parse(
-      const PCaselessString & auth,
-      BOOL proxy
+    virtual PBoolean Parse(
+      const PString & auth,
+      PBoolean proxy
     );
 
-    BOOL IsValid() const;
-
-    BOOL Authorise(
+    virtual PBoolean Authorise(
       SIP_PDU & pdu
     ) const;
 
+    virtual PString GetAuthRealm() const         { return authRealm; }
+    virtual void SetAuthRealm(const PString & r) { authRealm = r; }
+
     enum Algorithm {
       Algorithm_MD5,
       NumAlgorithms
     };
-
-    BOOL IsProxy() const                   { return isProxy; }
-    const PString & GetAuthRealm() const   { return authRealm; }
-    const PString & GetUsername() const    { return username; }
-    const PString & GetPassword() const    { return password; }
     const PString & GetNonce() const       { return nonce; }
     Algorithm GetAlgorithm() const         { return algorithm; }
     const PString & GetOpaque() const      { return opaque; }
 
-    void SetUsername(const PString & user) { username = user; }
-    void SetPassword(const PString & pass) { password = pass; }
-    void SetAuthRealm(const PString & r)   { authRealm = r; }
-
   protected:
-    BOOL      isProxy;
     PString   authRealm;
-    PString   username;
-    PString   password;
     PString   nonce;
     Algorithm algorithm;
-		PString   opaque;
+    PString   opaque;
 
-		BOOL qopAuth;
-		BOOL qopAuthInt;
-		PString cnonce;
-		mutable PAtomicInteger nonceCount;
+    PBoolean qopAuth;
+    PBoolean qopAuthInt;
+    PString cnonce;
+    mutable PAtomicInteger nonceCount;
 };
 
-
 /////////////////////////////////////////////////////////////////////////
 // SIP_PDU
 
@@ -535,9 +526,9 @@ class SIPAuthentication : public PObject
 	Class provides methods for reading from and writing to transport.
  */
 
-class SIP_PDU : public PObject
+class SIP_PDU : public PSafeObject
 {
-  PCLASSINFO(SIP_PDU, PObject);
+  PCLASSINFO(SIP_PDU, PSafeObject);
   public:
     enum Methods {
       Method_INVITE,
@@ -550,11 +541,17 @@ class SIP_PDU : public PObject
       Method_NOTIFY,
       Method_REFER,
       Method_MESSAGE,
+      Method_INFO,
+      Method_PING,
+      Method_PUBLISH,
       NumMethods
     };
 
     enum StatusCodes {
       IllegalStatusCode,
+      Local_TransportError,
+      Local_BadTransportAddress,
+      Local_Timeout,
 
       Information_Trying                  = 100,
       Information_Ringing                 = 180,
@@ -599,7 +596,7 @@ class SIP_PDU : public PObject
       Failure_BusyHere                    = 486,
       Failure_RequestTerminated           = 487,
       Failure_NotAcceptableHere           = 488,
-      Failure_BadEvent			          = 489,
+      Failure_BadEvent                    = 489,
       Failure_RequestPending              = 491,
       Failure_Undecipherable              = 493,
 
@@ -619,7 +616,8 @@ class SIP_PDU : public PObject
       MaxStatusCode                       = 699
     };
 
-	static const char * GetStatusCodeDescription (int code);
+	static const char * GetStatusCodeDescription(int code);
+    friend ostream & operator<<(ostream & strm, StatusCodes status);
 
     enum {
       MaxSize = 65535
@@ -653,7 +651,8 @@ class SIP_PDU : public PObject
       const SIP_PDU & request,
       StatusCodes code,
       const char * contact = NULL,
-      const char * extra = NULL
+      const char * extra = NULL,
+      const SDPSessionDescription * sdp = NULL
     );
     SIP_PDU(const SIP_PDU &);
     SIP_PDU & operator=(const SIP_PDU &);
@@ -681,42 +680,55 @@ class SIP_PDU : public PObject
       const OpalTransport & transport
     );
 
-    /**Add and populate Route header if connection has routeSet.
-	   If first route is strict, exchange with URI.
-	   Returns TRUE if conection has a routeSet.
-	  */
-    BOOL SetRoute(SIPConnection & connection);
+    /**Add and populate Route header following the given routeSet.
+      If first route is strict, exchange with URI.
+      Returns PTrue if routeSet.
+      */
+    bool SetRoute(const PStringList & routeSet);
+    bool SetRoute(const SIPURL & proxy);
 
     /**Set mime allow field to all supported methods.
       */
-    void SetAllow(void);
+    void SetAllow(unsigned bitmask);
 
     /**Update the VIA field following RFC3261, 18.2.1 and RFC3581.
       */
     void AdjustVia(OpalTransport & transport);
     
-    /**Return the address from the via field. That address
-     * should be used to send responses to incoming PDUs.
-     */
-    OpalTransportAddress GetViaAddress(OpalEndPoint &);
-    
-    /**Return the address to which the request PDU should be sent
-     * according to the RFC, for a request in a dialog.
-     */
-    OpalTransportAddress GetSendAddress(SIPConnection &);
-    
     /**Read PDU from the specified transport.
       */
-    BOOL Read(
+    PBoolean Read(
       OpalTransport & transport
     );
 
     /**Write the PDU to the transport.
       */
-    BOOL Write(
-      OpalTransport & transport
+    PBoolean Write(
+      OpalTransport & transport,
+      const OpalTransportAddress & remoteAddress = OpalTransportAddress(),
+      const PString & localInterface = PString::Empty()
     );
 
+    /**Write PDU as a response to a request.
+    */
+    bool SendResponse(
+      OpalTransport & transport,
+      StatusCodes code,
+      SIPEndPoint * endpoint = NULL,
+      const char * contact = NULL,
+      const char * extra = NULL
+    );
+    bool SendResponse(
+      OpalTransport & transport,
+      SIP_PDU & response,
+      SIPEndPoint * endpoint = NULL
+    );
+
+    /** Construct the PDU string to output.
+        Returns the total length of the PDU.
+      */
+    PString Build();
+
     PString GetTransactionID() const;
 
     Methods GetMethod() const                { return method; }
@@ -729,13 +741,11 @@ class SIP_PDU : public PObject
     const PString & GetInfo() const          { return info; }
     const SIPMIMEInfo & GetMIME() const      { return mime; }
           SIPMIMEInfo & GetMIME()            { return mime; }
-    BOOL HasSDP() const                      { return sdp != NULL; }
-    SDPSessionDescription & GetSDP() const   { return *PAssertNULL(sdp); }
-    void SetSDP(SDPSessionDescription * s)   { sdp = s; }
-    void SetSDP(const SDPSessionDescription & s) { sdp = new SDPSessionDescription(s); }
+    void SetURI(const SIPURL & newuri)       { uri = newuri; }
+    SDPSessionDescription * GetSDP();
+    void SetSDP(SDPSessionDescription * sdp);
 
   protected:
-    
     Methods     method;                 // Request type, ==NumMethods for Response
     StatusCodes statusCode;
     SIPURL      uri;                    // display name & URI, no tag
@@ -745,13 +755,75 @@ class SIP_PDU : public PObject
     SIPMIMEInfo mime;
     PString     entityBody;
 
-    SDPSessionDescription * sdp;
+    SDPSessionDescription * m_SDP;
+
+    mutable PString transactionID;
 };
 
 
 PQUEUE(SIP_PDU_Queue, SIP_PDU);
 
 
+#if PTRACING
+ostream & operator<<(ostream & strm, SIP_PDU::Methods method);
+#endif
+
+
+/////////////////////////////////////////////////////////////////////////
+// SIPDialogContext
+
+/** Session Initiation Protocol dialog context information.
+  */
+class SIPDialogContext
+{
+  public:
+    SIPDialogContext();
+
+    const PString & GetCallID() const { return m_callId; }
+    void SetCallID(const PString & id) { m_callId = id; }
+
+    const SIPURL & GetRequestURI() const { return m_requestURI; }
+    void SetRequestURI(const SIPURL & url) { m_requestURI = url; }
+    bool SetRequestURI(const PString & uri) { return m_requestURI.Parse(uri); }
+
+    const PString & GetLocalTag() const { return m_localTag; }
+    void SetLocalTag(const PString & tag) { m_localTag = tag; }
+
+    const SIPURL & GetLocalURI() const { return m_localURI; }
+    void SetLocalURI(const SIPURL & url);
+    bool SetLocalURI(const PString & uri);
+
+    const PString & GetRemoteTag() const { return m_remoteTag; }
+    void SetRemoteTag(const PString & tag) { m_remoteTag = tag; }
+
+    const SIPURL & GetRemoteURI() const { return m_remoteURI; }
+    void SetRemoteURI(const SIPURL & url);
+    bool SetRemoteURI(const PString & uri);
+
+    const PStringList & GetRouteSet() const { return m_routeSet; }
+    void SetRouteSet(const PStringList & routes) { m_routeSet = routes; }
+    void UpdateRouteSet(const SIPURL & proxy);
+
+    void Update(const SIP_PDU & response);
+
+    unsigned GetNextCSeq(unsigned inc = 1) { return m_lastSentCSeq += inc; }
+    bool IsDuplicateCSeq(unsigned sequenceNumber);
+
+    bool IsEstablished() const { return !m_remoteTag.IsEmpty(); }
+
+  protected:
+    PString     m_callId;
+    SIPURL      m_requestURI;
+    SIPURL      m_localURI;
+    PString     m_localTag;
+    SIPURL      m_remoteURI;
+    PString     m_remoteTag;
+    PStringList m_routeSet;
+    unsigned    m_lastSentCSeq;
+    unsigned    m_lastReceivedCSeq;
+};
+
+
 /////////////////////////////////////////////////////////////////////////
 // SIPTransaction
 
@@ -785,28 +857,39 @@ class SIPTransaction : public SIP_PDU
     );
     ~SIPTransaction();
 
-    BOOL Start();
-    BOOL IsInProgress() const { return state == Trying && state == Proceeding; }
-    BOOL IsFailed() const { return state > Terminated_Success; }
-    BOOL IsFinished()     { return finished.Wait(0); }
-    BOOL IsCanceled() const {return state == Terminated_Cancelled;}
-    void Wait();
-    BOOL SendCANCEL();
+    PBoolean Start();
+    bool IsTrying()     const { return state == Trying; }
+    bool IsProceeding() const { return state == Proceeding; }
+    bool IsInProgress() const { return state == Trying || state == Proceeding; }
+    bool IsFailed()     const { return state > Terminated_Success; }
+    bool IsCompleted()  const { return state >= Completed; }
+    bool IsCanceled()   const { return state == Cancelling || state == Terminated_Cancelled || state == Terminated_Aborted; }
+    bool IsTerminated() const { return state >= Terminated_Success; }
 
-    virtual BOOL OnReceivedResponse(SIP_PDU & response);
-    virtual BOOL OnCompleted(SIP_PDU & response);
+    void WaitForCompletion();
+    PBoolean Cancel();
+    void Abort();
+
+    virtual PBoolean OnReceivedResponse(SIP_PDU & response);
+    virtual PBoolean OnCompleted(SIP_PDU & response);
 
     OpalTransport & GetTransport() const  { return transport; }
     SIPConnection * GetConnection() const { return connection; }
+    PString         GetInterface() const { return m_localInterface; }
 
-    const OpalTransportAddress & GetLocalAddress() const { return localAddress; }
-
+    static PString GenerateCallID();
+    
   protected:
     void Construct(
       const PTimeInterval & minRetryTime = PMaxTimeInterval,
       const PTimeInterval & maxRetryTime = PMaxTimeInterval
     );
-    BOOL ResendCANCEL();
+    void Construct(
+      Methods method, 
+      SIPDialogContext & dialog
+    );
+    bool SendPDU(SIP_PDU & pdu);
+    bool ResendCANCEL();
 
     PDECLARE_NOTIFIER(PTimer, SIPTransaction, OnRetry);
     PDECLARE_NOTIFIER(PTimer, SIPTransaction, OnTimeout);
@@ -822,33 +905,27 @@ class SIPTransaction : public SIP_PDU
       Terminated_RetriesExceeded,
       Terminated_TransportError,
       Terminated_Cancelled,
+      Terminated_Aborted,
       NumStates
     };
     virtual void SetTerminated(States newState);
 
-    SIPEndPoint   & endpoint;
-    OpalTransport & transport;
-    SIPConnection * connection;
+    SIPEndPoint           & endpoint;
+    OpalTransport         & transport;
+    PSafePtr<SIPConnection> connection;
+    PTimeInterval           retryTimeoutMin; 
+    PTimeInterval           retryTimeoutMax; 
 
-    States   state;
-    unsigned retry;
-    PTimer   retryTimer;
-    PTimer   completionTimer;
-
-    PSyncPoint finished;
-    PTimedMutex mutex;
-
-    PTimeInterval retryTimeoutMin; 
-    PTimeInterval retryTimeoutMax; 
-
-    OpalTransportAddress localAddress;
+    States     state;
+    unsigned   retry;
+    PTimer     retryTimer;
+    PTimer     completionTimer;
+    PSyncPoint completed;
+    PString              m_localInterface;
+    OpalTransportAddress m_remoteAddress;
 };
 
 
-PLIST(SIPTransactionList, SIPTransaction);
-PDICTIONARY(SIPTransactionDict, PString, SIPTransaction);
-
-
 /////////////////////////////////////////////////////////////////////////
 // SIPInvite
 
@@ -861,27 +938,19 @@ class SIPInvite : public SIPTransaction
 {
     PCLASSINFO(SIPInvite, SIPTransaction);
   public:
-    SIPInvite(
-      SIPConnection & connection,
-      OpalTransport & transport
-    );
     SIPInvite(
       SIPConnection & connection,
       OpalTransport & transport,
-      RTP_SessionManager & sm
-    );
-    SIPInvite(
-      SIPConnection & connection,
-      OpalTransport & transport,
-      unsigned rtpSessionId
+      const OpalRTPSessionManager & sm
     );
 
-    virtual BOOL OnReceivedResponse(SIP_PDU & response);
+    virtual PBoolean OnReceivedResponse(SIP_PDU & response);
 
-    RTP_SessionManager & GetSessionManager() { return rtpSessions; }
+    const OpalRTPSessionManager & GetSessionManager() const { return rtpSessions; }
+          OpalRTPSessionManager & GetSessionManager()       { return rtpSessions; }
 
   protected:
-    RTP_SessionManager rtpSessions;
+    OpalRTPSessionManager rtpSessions;
 };
 
 
@@ -891,42 +960,154 @@ class SIPRegister : public SIPTransaction
 {
     PCLASSINFO(SIPRegister, SIPTransaction);
   public:
+    enum CompatibilityModes {
+      e_FullyCompliant,                 /**< Registrar is fully compliant, we will register
+                                             all listeners of all types (e.g. sip, sips etc)
+                                             in the Contact field. */
+      e_CannotRegisterMultipleContacts, /**< Registrar refuses to register more than one
+                                             contact field. Correct behaviour is to return
+                                             a contact with the fields it can accept in
+                                             the 200 OK */
+      e_CannotRegisterPrivateContacts   /**< Registrar refuses to register any RFC
+                                             contact field. Correct behaviour is to return
+                                             a contact with the fields it can accept in
+                                             the 200 OK */
+    };
+
+    /// Registrar parameters
+    struct Params {
+      Params();
+
+      PString       m_addressOfRecord;
+      PString       m_registrarAddress;
+      PString       m_contactAddress;
+      PString       m_authID;
+      PString       m_password;
+      PString       m_realm;
+      unsigned      m_expire;
+      unsigned      m_restoreTime;
+      PTimeInterval m_minRetryTime;
+      PTimeInterval m_maxRetryTime;
+      void        * m_userData;
+      CompatibilityModes m_compatibility;
+    };
+
     SIPRegister(
       SIPEndPoint   & endpoint,
       OpalTransport & transport,
-      const SIPURL & address,
-      const PString & id,
-      unsigned expires,
-      const PTimeInterval & minRetryTime = PMaxTimeInterval,
-      const PTimeInterval & maxRetryTime = PMaxTimeInterval
+      const SIPURL & proxy,
+      const PString & callId,
+      unsigned cseq,
+      const Params & params
     );
 };
 
 
 /////////////////////////////////////////////////////////////////////////
 
-class SIPMWISubscribe : public SIPTransaction
+class SIPSubscribe : public SIPTransaction
 {
-    PCLASSINFO(SIPMWISubscribe, SIPTransaction);
+    PCLASSINFO(SIPSubscribe, SIPTransaction);
   public:
-   /** Valid types for a MWI
+    /** Valid types for a presence event
      */
-    enum MWIType { 
-      
-      VoiceMessage, 
-      FaxMessage, 
-      PagerMessage, 
-      MultimediaMessage, 
-      TextMessage, 
-      None 
+    enum PredefinedPackages {
+      MessageSummary,
+      Presence,
+      Dialog,
+      NumPredefinedPackages
     };
 
-  SIPMWISubscribe(
-      SIPEndPoint   & endpoint,
-      OpalTransport & transport,
-      const SIPURL & address,
+    class EventPackage : public PCaselessString
+    {
+        PCLASSINFO(EventPackage, PCaselessString);
+      public:
+        EventPackage(PredefinedPackages);
+        EventPackage(const PString & str) : PCaselessString(str) { }
+        EventPackage(const char * cstr) : PCaselessString(cstr) { }
+        bool operator==(PredefinedPackages) const;
+        bool operator==(const PString & str) const { return Compare(str) == EqualTo; }
+        bool operator==(const char * cstr) const { return InternalCompare(0, P_MAX_INDEX, cstr) == EqualTo; }
+        virtual Comparison InternalCompare(PINDEX offset, PINDEX length, const char * cstr) const;
+    };
+
+    struct Params {
+      Params(PredefinedPackages pkg = NumPredefinedPackages);
+
+      EventPackage  m_eventPackage;
+      PString       m_agentAddress;
+      PString       m_addressOfRecord;
+      PString       m_contactAddress;
+      PString       m_authID;
+      PString       m_password;
+      PString       m_realm;
+      unsigned      m_expire;
+      unsigned      m_restoreTime;
+      PTimeInterval m_minRetryTime;
+      PTimeInterval m_maxRetryTime;
+      void          * m_userData;
+      PString       m_from;
+    };
+
+    SIPSubscribe(
+        SIPEndPoint & ep,
+        OpalTransport & trans,
+        SIPDialogContext & dialog,
+        const Params & params
+    );
+};
+
+
+typedef SIPSubscribe::EventPackage SIPEventPackage;
+
+
+/////////////////////////////////////////////////////////////////////////
+
+class SIPHandler;
+
+class SIPEventPackageHandler
+{
+public:
+  virtual ~SIPEventPackageHandler() { }
+  virtual PCaselessString GetContentType() const = 0;
+  virtual bool OnReceivedNOTIFY(SIPHandler & handler, SIP_PDU & request) = 0;
+  virtual PString OnSendNOTIFY(SIPHandler & /*handler*/, const PObject * /*body*/) { return PString::Empty(); }
+};
+
+
+typedef PFactory<SIPEventPackageHandler, SIPEventPackage> SIPEventPackageFactory;
+
+
+/////////////////////////////////////////////////////////////////////////
+
+class SIPNotify : public SIPTransaction
+{
+    PCLASSINFO(SIPNotify, SIPTransaction);
+  public:
+    SIPNotify(
+        SIPEndPoint & ep,
+        OpalTransport & trans,
+        SIPDialogContext & dialog,
+        const SIPEventPackage & eventPackage,
+        const PString & state,
+        const PString & body
+    );
+};
+
+
+/////////////////////////////////////////////////////////////////////////
+
+class SIPPublish : public SIPTransaction
+{
+    PCLASSINFO(SIPPublish, SIPTransaction);
+  public:
+    SIPPublish(
+      SIPEndPoint & ep,
+      OpalTransport & trans,
       const PString & id,
-      unsigned expires
+      const PString & sipIfMatch,
+      SIPSubscribe::Params & params,
+      const PString & body
     );
 };
 
@@ -940,20 +1121,20 @@ class SIPRefer : public SIPTransaction
     SIPRefer(
       SIPConnection & connection,
       OpalTransport & transport,
-      const PString & refer
+      const SIPURL & refer
     );
     SIPRefer(
       SIPConnection & connection,
       OpalTransport & transport,
-      const PString & refer,
-      const PString & referred_by
+      const SIPURL & refer,
+      const SIPURL & referred_by
     );
   protected:
     void Construct(
       SIPConnection & connection,
       OpalTransport & transport,
-      const PString & refer,
-      const PString & referred_by = PString::Empty()
+      const SIPURL & refer,
+      const SIPURL & referred_by
     );
 };
 
@@ -985,10 +1166,12 @@ class SIPMessage : public SIPTransaction
     
   public:
     SIPMessage(
-	       SIPEndPoint & ep,
-	       OpalTransport & trans,
-	       const SIPURL & address,
-	       const PString & body
+      SIPEndPoint & ep,
+      OpalTransport & trans,
+      const SIPURL & proxy,
+      const SIPURL & to,
+      const PString & id,
+      const PString & body
     );
 };
 
@@ -1002,20 +1185,10 @@ class SIPAck : public SIP_PDU
 {
     PCLASSINFO(SIPAck, SIP_PDU);
   public:
-    // This ACK is sent for non-2xx responses
     SIPAck(
-      SIPEndPoint & ep,
       SIPTransaction & invite,
-      SIP_PDU & response); 
-
-    // This ACK is sent for 2xx responses according to 17.1.1.3
-    SIPAck(
-      SIPTransaction & invite);
-
-  protected:
-    void Construct();
-
-    SIPTransaction & transaction;
+      SIP_PDU & response
+    );
 };
 
 
@@ -1029,14 +1202,34 @@ class SIPOptions : public SIPTransaction
     
   public:
     SIPOptions(
-      SIPEndPoint & ep,
+        SIPEndPoint & ep,
       OpalTransport & trans,
-      const SIPURL & address
+       const SIPURL & address
     );
 };
 
 
-#endif // __OPAL_SIPPDU_H
+/////////////////////////////////////////////////////////////////////////
+
+/* This is a PING PDU, with a body
+ */
+class SIPPing : public SIPTransaction
+{
+  PCLASSINFO(SIPPing, SIPTransaction);
+
+  public:
+    SIPPing(
+               SIPEndPoint & ep,
+             OpalTransport & trans,
+              const SIPURL & address,
+              const PString & body = PString::Empty()
+   );
+};
+
+
+#endif // OPAL_SIP
+
+#endif // OPAL_SIP_SIPPDU_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
